c Searches for maximum peak in box when smoothed with a top-hat filter
c
      program peakfindfft
c  
      include 'peakfind.inc'
      parameter(Lmax=Ls,Lmax3=Lmax**3,Lmaxc3=(Lmax/2+1)**3)
c
      real rc(3),rs(3),good_radius,gp3(Lmaxc3),d3s(Ls,Ls,Ls),
     &     zi(ndim+1,ndim),lambda,xi(ndim,ndim),temp(ndim),
     &     y(ndim+1),tpi,ftol,FUNC
      integer mass,ihoc,ll,np,mp,nobj,foundmass
c      character default*9
      integer default
c
c Variables for endless loop flags - see below
      real oldrad,olddelta,oldrs(3)
      integer rflag,dflag,rsflag
c
      common/rv/r(3,nobmax),d3(Ls,Ls,Ls),
     &     ihoc(Ls**3),ll(nobmax),radius
      common/ln/L,nobj
c
      external func
c
      tpi=6.283185307
      pi=tpi/2.
      ftol=0.0001
      np=ndim
      mp=np+1
c
c  Read particle data from r.pert
c  Particle positions go from 1 to L+1
c  nobj particles in total
c  rad0 is 8/h Mpc / boxsize
c  sigma8 is desired sigma8 from cosmic
      open(unit=1,file='data/r.pert',status='old',form='unformatted',
     &     iostat=ier)
      if (ier.ne.0) STOP 'error opening r.pert'
      read(1) L,nobj,rad0,sigma8
      if (L.ne.Ls) STOP 'recompile with Ls=L'
      rL=float(L)
      rL1=rL-1.
      if (nobj.gt.nobmax) STOP 'nobj>nobmax'
      write(*,*) 'Desired sigma8 in r.pert = ',sigma8
      rhomean=float(nobj)/float(L**3)
      CALL readr(1,nobj,r,ier)
      if (ier.ne.0) STOP 'error reading r.pert'
      close(unit=1)
c
c Smooth particles onto mesh.
      call assign
      do 11 iz=1,Ls
         do 11 iy=1,Ls
            do 11 ix=1,Ls
 11   d3s(ix,iy,iz)=d3(ix,iy,iz)        
c
c      write(default,*) nint(4.*pi/3.*(rad0*L)**3)
c 232  CALL getint(mass,'mass of peak',default,imode)
      default=nint(4.*pi/3.*nobj*rad0**3)
 232  write(*,*) 'Enter mass of peak (',default,')'
      read(*,*) mass
      if (mass.eq.0) mass=default
      if (mass.gt.nobj) then
         print*,'Sphere mass must be smaller than universe mass!'
         goto 232
      end if
      rad0=(3./4./pi*float(mass)/rhomean)**(1./3.)
      step=0.5*rad0
c
c Flags for the endless loop problem - see below 
      oldrad=-1.e32
      olddelta=-1.e32
      do 33 i=1,3
 33      oldrs(i)=-1.e32
      rsflag=0
      dflag=0
      rflag=0
c
c Make unit vectors (xi) to be used in powell and amoeba
      lambda=step
      do i=1,ndim
         do j=1,ndim
            if (i.eq.j) xi(i,j)=step
            if (i.ne.j) xi(i,j)=0.
         end do
      end do
c
c First search on a grid to find the most likely starting point for
c the refined peak search:
c   Create smoothed density field:
      call wthgreen(rad0,L,gp3)
      do iz=1,Ls
         do iy=1,Ls
            do ix=1,Ls
               d3(ix,iy,iz)=d3s(ix,iy,iz)        
            end do
         end do
      end do
      call cnvl(d3,L,gp3)
c   Search for maximum
      radius=rad0
      d2sum=0.
      dmax=0.
      gstep=2**(int(log10(rad0)/log10(2.)))
      nstep=nint(float(Ls)/gstep)
      do while (nstep.lt.16 .and. nstep.lt.L) 
         gstep=gstep/2.
         nstep=nstep*2
      end do
      do iz=1,nstep
         rc(3)=1.+(iz-1)*gstep
         do iy=1,nstep
            rc(2)=1.+(iy-1)*gstep
            do ix=1,nstep
               rc(1)=1.+(ix-1)*gstep
               dens=deltafft(rc)/rhomean
               d2sum=d2sum+dens**2
               if (dens.gt.dmax) then
                  dmax=dens
                  rs(1)=rc(1)
                  rs(2)=rc(2)
                  rs(3)=rc(3)
               end if
            end do
         end do
      end do
      sigma=sqrt(d2sum/float(nstep**3))
      write(*,*) 'Finished grid search'
      write(*,*) 'rms delta =',sigma
      write(*,*) 'maximum delta =',dmax
      write(*,*) 'position = ',rs
c
c Now do refined search for peak position and size.
c This repeat loop goes until a correct size of radius for the
c mass is found for the densest region near the input co-ord.
c The minimization routine can be chosen to be either powell,
c amoeba or Hugh's method. Powell and amoeba both come from Num. Rec..
c 
 29   radius = good_radius(mass,oldrad,rs,ftol,rflag)
      write(*,*) 'New radius = ',radius
      oldrad=radius
      av_mass=4.*pi/3. * radius**3
      call wthgreen(radius,L,gp3)
      do 12 iz=1,Ls
         do 12 iy=1,Ls
            do 12 ix=1,Ls
 12   d3(ix,iy,iz)=d3s(ix,iy,iz)        
      call cnvl(d3,L,gp3)

c------------------------------------------------
c Amoeba (Downhill Simplex Minimization)
c the initial simplex is built around the input rs points using an
c arbitrary constant 'lambda' which is defined above
c and which represents a guess at the problem's characteristic
c length scale.
c the routine returns ndim+1 minimum points (not just one) which defines
c the simplex that is within the tolerance of a minimum. The minimum of
c these points is used as the overall minimum.
      do i=1,ndim
         zi(1,i)=rs(i)
      enddo
      do i=2,ndim+1
         do j=1,ndim
            if (j.eq.i-1) then
               zi(i,j)=rs(j)+lambda
            else
               zi(i,j)=rs(j)
            endif
         enddo
      enddo
      do i=1,ndim+1
         do j=1,ndim
            temp(j)=zi(i,j)
         enddo
         y(i)=func(temp)
      enddo
      call amoeba(zi,y,mp,np,ndim,ftol,func,iter)
      delta=float(nobmax)
      do k=1,ndim+1
         delta=min(delta,y(k))
         if (delta.eq.y(k)) m=k
      enddo
      do i=1,3
          rs(i)=mod(zi(m,i)+rL1,rL)+1.
      end do
      delta=-delta

c------------------------------------------------

c The following commands eliminate the possibility of endless looping
c which might occur if the routines find a maximum but are unable to meet
c the mass accuracy criterion.
c The iteration is stopped if the 3 variables do not change from one iteration
c to another.
c
      if (delta.eq.olddelta) dflag=1 
      olddelta=delta
      do i=1,3
         if (rs(i).eq.oldrs(i)) rsflag=rsflag+1
         oldrs(i)=rs(i)
      end do
      if ((rflag.eq.1).and.(dflag.eq.1).and.(rsflag.eq.3)) goto 147
      rsflag=0
c
      foundmass=nsphere(rs)
      if (abs(float(foundmass)/float(mass)-1.).ge.(5.*ftol)) then
         write(*,*) 'New position = ',rs
         write(*,*) 'Delta = ',delta/rhomean
         goto 29
      end if
c
 147  print*,'Center of top hat : ',rs
      print*,'radius = ',radius
      print*,'Average mass = ',av_mass*rhomean
      print*,'requested mass = ',mass
      print*,' mass  =    ',foundmass
      print*,'Overdensity, delta, = ',delta/rhomean
      print*,'delta/sigma = ',delta/rhomean/sigma
c
      open(7,file='peak.dat',status='unknown')
      write(7,*) rs,delta
      close(7)
c
      end

c=============================================

      SUBROUTINE amoeba(p,y,mp,np,ndim,ftol,func,iter)

      INTEGER iter,mp,ndim,np,NMAX,ITMAX
      REAL ftol,p(mp,np),y(mp),func
      PARAMETER (NMAX=20,ITMAX=5000)
      INTEGER i,ihi,ilo,inhi,j,m,n
      REAL rtol,sum,swap,ysave,ytry,psum(NMAX),amotry
      external func
      iter=0
 1    do n=1,ndim
         sum=0.
         do m=1,ndim+1
            sum=sum+p(m,n)
         enddo
         psum(n)=sum
      enddo
 2    ilo=1
      if (y(1).gt.y(2)) then
         ihi=1
         inhi=2
      else
         ihi=2
         inhi=1
      endif
      do i=1,ndim+1
         if(y(i).le.y(ilo)) ilo=i
         if(y(i).gt.y(ihi)) then
            inhi=ihi
            ihi=i
         else if(y(i).gt.y(inhi)) then
            if(i.ne.ihi) inhi=i
         endif
      enddo
      rtol=2.*abs(y(ihi)-y(ilo))/(abs(y(ihi))+abs(y(ilo)))
      if (rtol.lt.ftol) then
         swap=y(1)
         y(1)=y(ilo)
         y(ilo)=swap
         do n=1,ndim
            swap=p(1,n)
            p(1,n)=p(ilo,n)
            p(ilo,n)=swap
         enddo
         return
      endif
      if (iter.ge.ITMAX) pause 'ITMAX exceeded'
      iter=iter+2
      ytry=amotry(p,y,psum,mp,np,ndim,func,ihi,-1.0)
      if (ytry.le.y(ilo)) then
         ytry=amotry(p,y,psum,mp,np,ndim,func,ihi,2.0)
      else if (ytry.ge.y(inhi)) then
         ysave=y(ihi)
         ytry=amotry(p,y,psum,mp,np,ndim,func,ihi,0.5)
         if (ytry.ge.ysave) then
            do i=1,ndim+1
               if(i.ne.ilo)then
                  do j=1,ndim
                     psum(j)=0.5*(p(i,j)+p(ilo,j))
                     p(i,j)=psum(j)
                  enddo
                  y(i)=func(psum)
               endif
            enddo
            iter=iter+ndim
            goto 1
         endif
      else
         iter=iter-1
      endif
      goto 2
      END

c====================================================

      subroutine assign

      include 'peakfind.inc'
      common/rv/r(3,nobmax),d3(Ls,Ls,Ls),
     &     ihoc(Ls**3),ll(nobmax),radius
      common/ln/L,nobj
c
      rca=1.0
      rcb=0.0
      rL=float(L)
      N=nobj
      Ls2=Ls*Ls
      Lscube=Ls**3

      do 1 i=1,Lscube
 1    ihoc(i)=0

      do 11 i=1,nobmax
 11   ll(i)=0

      do 12 ix=1,L
         do 12 iy=1,L
            do 12 iz=1,L
 12   d3(ix,iy,iz)=0.

      do 2 i=1,N
         rrx=r(1,i)
         rry=r(2,i)
         r3i=r(3,i)
         rrz=r3i
         ix= int(rrx)
         iy= int(rry)
         iz= int(rrz)
         ibox=ix+Ls*(iy-1)+Ls2*(iz-1)
c-------------------------------------
c         ins=ihoc(ibox)
c         insp=ins
c         if(r(3,i).ge.r(3,ins))then
c            ll(i)=ins
c            ihoc(ibox)=i
c         else
c 222        if(r(3,i).lt.r(3,ins) .and. ins.ne.0)then
c               insp=ins
c               ins=ll(ins)
c               goto 222
c            end if
c            ll(insp)=i
c            ll(i)=ins
c         end if
c------------------------------------
         inst=ihoc(ibox)
         ins=inst
         insp=inst

 222     if(ins.gt.0)then
            if(r3i.lt.r(3,ins))then
               insp=ins
               ins=ll(ins)
               goto 222
            end if
         end if
         ll(i)=ins
         if(ins.eq.inst)then
            ihoc(ibox)=i
         else
            ll(insp)=i
         end if
c--------------------------------------
         ix= int(rrx+0.5)
         iy= int(rry+0.5)
         iz= int(rrz+0.5)
         hx = rrx -ix
         hy = rry -iy
         hz = rrz -iz
         ix=mod(ix-1,L)+1
         iy=mod(iy-1,L)+1
         iz=mod(iz-1,L)+1
         hx0=0.75 - hx*hx
         hxp1=0.5* (0.5 + hx)**2
         hxm1=0.5* (0.5 - hx)**2
         hy0=0.75 - hy*hy
         hyp1=0.5* (0.5 + hy)**2
         hym1=0.5* (0.5 - hy)**2
         hz0= 0.75 - hz*hz
         hzp1=0.5* (0.5 + hz)**2
         hzm1=0.5* (0.5 - hz)**2
         ixp1=mod(ix,L)+1
         iyp1=mod(iy,L)+1
         izp1=mod(iz,L)+1
         ixm1=mod(ix-2+L,L)+1
         iym1=mod(iy-2+L,L)+1
         izm1=mod(iz-2+L,L)+1
         d3(ixm1,iym1,izm1)   = d3(ixm1,iym1,izm1)+ hxm1*hym1 *hzm1
         d3(ix  ,iym1,izm1)   = d3(ix  ,iym1,izm1)+ hx0 *hym1 *hzm1
         d3(ixp1,iym1,izm1)   = d3(ixp1,iym1,izm1)+ hxp1*hym1 *hzm1
         d3(ixm1,iy  ,izm1)   = d3(ixm1,iy  ,izm1)+ hxm1*hy0  *hzm1
         d3(ix  ,iy  ,izm1)   = d3(ix  ,iy  ,izm1)+ hx0 *hy0  *hzm1
         d3(ixp1,iy  ,izm1)   = d3(ixp1,iy  ,izm1)+ hxp1*hy0  *hzm1
         d3(ixm1,iyp1,izm1)   = d3(ixm1,iyp1,izm1)+ hxm1*hyp1 *hzm1
         d3(ix  ,iyp1,izm1)   = d3(ix  ,iyp1,izm1)+ hx0 *hyp1 *hzm1
         d3(ixp1,iyp1,izm1)   = d3(ixp1,iyp1,izm1)+ hxp1*hyp1 *hzm1
         d3(ixm1,iym1,iz  )   = d3(ixm1,iym1,iz  )+ hxm1*hym1 *hz0
         d3(ix  ,iym1,iz  )   = d3(ix  ,iym1,iz  )+ hx0 *hym1 *hz0
         d3(ixp1,iym1,iz  )   = d3(ixp1,iym1,iz  )+ hxp1*hym1 *hz0
         d3(ixm1,iy  ,iz  )   = d3(ixm1,iy  ,iz  )+ hxm1*hy0  *hz0
         d3(ix  ,iy  ,iz  )   = d3(ix  ,iy  ,iz  )+ hx0 *hy0  *hz0
         d3(ixp1,iy  ,iz  )   = d3(ixp1,iy  ,iz  )+ hxp1*hy0  *hz0
         d3(ixm1,iyp1,iz  )   = d3(ixm1,iyp1,iz  )+ hxm1*hyp1 *hz0
         d3(ix  ,iyp1,iz  )   = d3(ix  ,iyp1,iz  )+ hx0 *hyp1 *hz0
         d3(ixp1,iyp1,iz  )   = d3(ixp1,iyp1,iz  )+ hxp1*hyp1 *hz0
         d3(ixm1,iym1,izp1)   = d3(ixm1,iym1,izp1)+ hxm1*hym1 *hzp1
         d3(ix  ,iym1,izp1)   = d3(ix  ,iym1,izp1)+ hx0 *hym1 *hzp1
         d3(ixp1,iym1,izp1)   = d3(ixp1,iym1,izp1)+ hxp1*hym1 *hzp1
         d3(ixm1,iy  ,izp1)   = d3(ixm1,iy  ,izp1)+ hxm1*hy0  *hzp1
         d3(ix  ,iy  ,izp1)   = d3(ix  ,iy  ,izp1)+ hx0 *hy0  *hzp1
         d3(ixp1,iy  ,izp1)   = d3(ixp1,iy  ,izp1)+ hxp1*hy0  *hzp1
         d3(ixm1,iyp1,izp1)   = d3(ixm1,iyp1,izp1)+ hxm1*hyp1 *hzp1
         d3(ix  ,iyp1,izp1)   = d3(ix  ,iyp1,izp1)+ hx0 *hyp1 *hzp1
         d3(ixp1,iyp1,izp1)   = d3(ixp1,iyp1,izp1)+ hxp1*hyp1 *hzp1
 2    continue
      
      return
      end

c====================================================

      function amotry(p,y,psum,mp,np,ndim,func,ihi,fac)

      integer ihi,mp,ndim,np,NMAX
      real amotry,fac,p(mp,np),psum(np),y(mp),func
      parameter (nmax=20)
      integer j
      real fac1,fac2,ytry,ptry(nmax)

      fac1=(1.-fac)/ndim
      fac2=fac1-fac
      do j=1,ndim
         ptry(j)=psum(j)*fac1-p(ihi,j)*fac2
      enddo
      ytry=func(ptry)
      if (ytry.lt.y(ihi)) then
         y(ihi)=ytry
         do j=1,ndim
            psum(j)=psum(j)-p(ihi,j)+ptry(j)
            p(ihi,j)=ptry(j)
         enddo
      endif
      amotry=ytry
      return
      end

c================================================

      integer function nsphere(rs)
c
      include 'peakfind.inc'
      real rs(3)
      common/rv/r(3,nobmax),d3(Ls,Ls,Ls),
     &     ihoc(Ls**3),ll(nobmax),radius
      common/ln/L,nobj
c
      rca=float(Ls)/float(L)
      rcb=1.-rca
      ria=1./rca
      rib=1.-ria
      rL=float(L)
      Ls2=Ls*Ls
      rLh=rL/2.
      rLf=10.5*rL
      Ls10=10*Ls
      radius2=radius**2.

      xc=rs(1)
      yc=rs(2)
      zc=rs(3)

      nin=0

      ixup=int((xc+radius)*rca+rcb)
      ixlow=int((xc-radius)*rca+rcb+200)-200
      iyup=int((yc+radius)*rca+rcb)
      iylow=int((yc-radius+200)*rca+rcb)-200
      izup=int((zc+radius)*rca+rcb)
      izlow=int((zc-radius+200)*rca+rcb)-200
      do 220 in=izlow,izup
         do 220 im=iylow,iyup
            do 220 il=ixlow,ixup
               ix=mod(il-1+Ls10,Ls)+1
               iy=mod(im-1+Ls10,Ls)+1
               iz=mod(in-1+Ls10,Ls)+1
               ibox=ix+Ls*(iy-1)+Ls2*(iz-1)
               i=ihoc(ibox)
 77            if(i.gt.0)then
                  xx=r(1,i)-xc
                  yy=r(2,i)-yc
                  zz=r(3,i)-zc
                  xx=mod(xx+rLf,rL)-rLh
                  yy=mod(yy+rLf,rL)-rLh
                  zz=mod(zz+rLf,rL)-rLh
                  rd2=xx**2+yy**2+zz**2
                  if(radius2.gt.rd2)nin=nin+1
                  i=ll(i)
                  goto 77
               end if

 220  continue
      nsphere=nin

      return
      end

c===============================================

      real function good_radius(mass,oldrad,rs,ftol,rflag)

      parameter(pi=3.141592654)
      include 'peakfind.inc'

      real rs(3),radius,floor,ceiling,ftol,oldrad
      integer nsphere,mass,sphere_mass,rflag
      
      common/rv/r(3,nobmax),d3(Ls,Ls,Ls),
     &     ihoc(Ls**3),ll(nobmax),radius

      floor= 0.
      ceiling = 2.*(((float(3*mass))/(4.*pi))**(1./3.))

 17   radius = (floor+ceiling)/2.
      sphere_mass = nsphere(rs)
      if(sphere_mass.ge.mass)then
         ceiling = radius
      else if(sphere_mass.lt.mass)then
         floor = radius
      end if
      
      if (abs((float(sphere_mass)/float(mass))-1.).gt.(5.*ftol))
     c     goto 17

      good_radius = radius
      if (good_radius.eq.oldrad) rflag=1

      return
      end

c===================================================

      real function deltafft(rc)
c
      include 'peakfind.inc'
      real rc(3)
      common/rv/r(3,nobmax),d3(Ls,Ls,Ls),
     &     ihoc(Ls**3),ll(nobmax),radius
      common/ln/L,nobj
c
      rrx=rc(1)
      rry=rc(2)
      rrz=rc(3)
      ix= int(rrx+0.5)
      iy= int(rry+0.5)
      iz= int(rrz+0.5)
      hx = rrx -ix
      hy = rry -iy
      hz = rrz -iz
      ix=mod(ix-1,L)+1
      iy=mod(iy-1,L)+1
      iz=mod(iz-1,L)+1

      hx0=0.75 - hx*hx
      hxp1=0.5* (0.5 + hx)**2
      hxm1=0.5* (0.5 - hx)**2
      hy0=0.75 - hy*hy
      hyp1=0.5* (0.5 + hy)**2
      hym1=0.5* (0.5 - hy)**2
      hz0=0.75 - hz*hz
      hzp1=0.5* (0.5 + hz)**2
      hzm1=0.5* (0.5 - hz)**2
      ixp1=mod(ix,L)+1
      iyp1=mod(iy,L)+1
      izp1=mod(iz,L)+1
      ixm1=mod(ix-2+L,L)+1
      iym1=mod(iy-2+L,L)+1
      izm1=mod(iz-2+L,L)+1
      ac =    d3(ixm1,iym1,izm1)* hxm1*hym1 *hzm1
     x       +d3(ix  ,iym1,izm1)* hx0 *hym1 *hzm1
     x       +d3(ixp1,iym1,izm1)* hxp1*hym1 *hzm1
     x       +d3(ixm1,iy  ,izm1)* hxm1*hy0  *hzm1
     x       +d3(ix  ,iy  ,izm1)* hx0 *hy0  *hzm1
     x       +d3(ixp1,iy  ,izm1)* hxp1*hy0  *hzm1
     x       +d3(ixm1,iyp1,izm1)* hxm1*hyp1 *hzm1
     x       +d3(ix  ,iyp1,izm1)* hx0 *hyp1 *hzm1
     x       +d3(ixp1,iyp1,izm1)* hxp1*hyp1 *hzm1
     x       +d3(ixm1,iym1,iz  )* hxm1*hym1 *hz0
     x       +d3(ix  ,iym1,iz  )* hx0 *hym1 *hz0
     x       +d3(ixp1,iym1,iz  )* hxp1*hym1 *hz0
     x       +d3(ixm1,iy  ,iz  )* hxm1*hy0  *hz0
     x       +d3(ix  ,iy  ,iz  )* hx0 *hy0  *hz0
      ac = ac
     x       +d3(ixp1,iy  ,iz  )* hxp1*hy0  *hz0
     x       +d3(ixm1,iyp1,iz  )* hxm1*hyp1 *hz0
     x       +d3(ix  ,iyp1,iz  )* hx0 *hyp1 *hz0
     x       +d3(ixp1,iyp1,iz  )* hxp1*hyp1 *hz0
     x       +d3(ixm1,iym1,izp1)* hxm1*hym1 *hzp1
     x       +d3(ix  ,iym1,izp1)* hx0 *hym1 *hzp1
     x       +d3(ixp1,iym1,izp1)* hxp1*hym1 *hzp1
     x       +d3(ixm1,iy  ,izp1)* hxm1*hy0  *hzp1
     x       +d3(ix  ,iy  ,izp1)* hx0 *hy0  *hzp1
     x       +d3(ixp1,iy  ,izp1)* hxp1*hy0  *hzp1
     x       +d3(ixm1,iyp1,izp1)* hxm1*hyp1 *hzp1
     x       +d3(ix  ,iyp1,izp1)* hx0 *hyp1 *hzp1
     x       +d3(ixp1,iyp1,izp1)* hxp1*hyp1 *hzp1

       deltafft=ac

      return
      end

c================================================

      subroutine wthgreen(rad,L,gp3)

      parameter(tpi=6.28318530717959,Lt=513,ipn=2)

      dimension gp3(L/2+1,L/2+1,L/2+1)
      real rks(Lt,-ipn:ipn),qs(Lt,-ipn:ipn),SA2(Lt)

c      W(x)=3.*(sin(x)-x*cos(x))/x**3

      if(L/2+1.gt.Lt)stop 'Array error in wthgreen'

      sf=2./float(L**3)

      tpiL=tpi/float(L)
      L2=L/2
      Lc=L/2+1
      
      af=2./15.
      do 200 ig=1,Lc
         rki=tpiL*float(ig-1)
         s2=sin(rki/2.)
         s22=s2**2
         SA2(ig)=(1.+s22*(af*s22-1.))**2
         ps=2.*s2
         do 200 n=-ipn,ipn
            rkin=rki-tpi*n
            rks(ig,n)=rkin
            q=1.
            if(rkin.ne.0.)q=ps/rkin
            qs(ig,n)=q**6
 200  continue

      do 400 igz=1,Lc
         do 400 igy=1,igz
            do 400 igx=1,igy
               SUM=0.
               do 500 nz=-ipn,ipn
                  rkzn=rks(igz,nz)
                  do 500 ny=-ipn,ipn
                     rkyn=rks(igy,ny)
                     do 500 nx=-ipn,ipn
                        rkxn=rks(igx,ny)
                        rk2n=rkxn*rkxn+rkyn*rkyn+rkzn*rkzn
                        Wt=1.
                        if(rk2n.gt.0.)Wt=W(rad*sqrt(rk2n))
                        SUM=SUM+Wt*qs(igz,nz)*qs(igy,ny)*qs(igx,nx)
 500           continue
               Sd=SA2(igx)*SA2(igy)*SA2(igz)
               gp3t=sf*SUM/Sd
               gp3(igx,igy,igz)=gp3t
               gp3(igx,igz,igy)=gp3t
               gp3(igy,igz,igx)=gp3t
               gp3(igy,igx,igz)=gp3t
               gp3(igz,igx,igy)=gp3t
               gp3(igz,igy,igx)=gp3t
 400  continue

      gp3(1,1,1)=0.

      return
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccc
      function w(x)
c
      if (x.lt.0.0) then
         STOP 'w: x<0'
      else if (x.le.1e-2) then
         w=1.-0.1*x**2
      else
         w=3.*(sin(x)-x*cos(x))/x**3
      end if
c
      RETURN
      end
c===============================================

      subroutine cnvl(d3,L,g3)

      dimension d3(L,L,L),g3(L/2+1,L/2+1,L/2+1)
      double precision tpiN,sr,cr,snr,cnr,tmp
      integer nnh(3)

      tpiN=6.28318530717959d0/float(L)
      snr=sin(tpiN)
      cnr=-2.0d0*sin(0.5d0*tpiN)**2
      nnh(1)=L/2
      nnh(2)=L
      nnh(3)=L
      call four21(d3,nnh,3,+1)

      sr=0.
      cr=1.
      do 100 ix=1,L/4+1
         icx=mod(L/2-ix+1,L/2)+1
         igx=L/2-ix+2
         inx=2*ix-1
         incx=2*icx-1
         S=sr
         C=cr
         do 101 iy=1,L/2+1
            icy=mod(L-iy+1,L)+1
            igy=iy
cdir$ IVDEP
            do 101 iz=1,L/2+1
               icz=mod(L-iz+1,L)+1
               igz=iz
               gsum=0.5*(g3(ix,igy,igz)+g3(igx,igy,igz))
               gdif=0.5*(g3(ix,igy,igz)-g3(igx,igy,igz))
               ga=gsum+S*gdif
               gb=C*gdif
               gc=gsum-S*gdif
               d000r=d3(inx,iy,iz)*ga+d3(incx+1,icy,icz)*gb
               d000i=d3(inx+1,iy,iz)*ga+d3(incx,icy,icz)*gb
               d100r=d3(incx,iy,iz)*gc+d3(inx+1,icy,icz)*gb
               d100i=d3(incx+1,iy,iz)*gc+d3(inx,icy,icz)*gb
               d010r=d3(inx,icy,iz)*ga+d3(incx+1,iy,icz)*gb
               d010i=d3(inx+1,icy,iz)*ga+d3(incx,iy,icz)*gb
               d110r=d3(incx,icy,iz)*gc+d3(inx+1,iy,icz)*gb
               d110i=d3(incx+1,icy,iz)*gc+d3(inx,iy,icz)*gb
               d001r=d3(inx,iy,icz)*ga+d3(incx+1,icy,iz)*gb
               d001i=d3(inx+1,iy,icz)*ga+d3(incx,icy,iz)*gb
               d101r=d3(incx,iy,icz)*gc+d3(inx+1,icy,iz)*gb
               d101i=d3(incx+1,iy,icz)*gc+d3(inx,icy,iz)*gb
               d011r=d3(inx,icy,icz)*ga+d3(incx+1,iy,iz)*gb
               d011i=d3(inx+1,icy,icz)*ga+d3(incx,iy,iz)*gb
               d111r=d3(incx,icy,icz)*gc+d3(inx+1,iy,iz)*gb
               d111i=d3(incx+1,icy,icz)*gc+d3(inx,iy,iz)*gb
               d3(inx,iy,iz)=d000r
               d3(inx+1,iy,iz)=d000i
               d3(incx,iy,iz)=d100r
               d3(incx+1,iy,iz)=d100i
               d3(inx,iy,icz)=d001r
               d3(inx+1,iy,icz)=d001i
               d3(incx,iy,icz)=d101r
               d3(incx+1,iy,icz)=d101i
               d3(inx,icy,iz)=d010r
               d3(inx+1,icy,iz)=d010i
               d3(incx,icy,iz)=d110r
               d3(incx+1,icy,iz)=d110i
               d3(inx,icy,icz)=d011r
               d3(inx+1,icy,icz)=d011i
               d3(incx,icy,icz)=d111r
               d3(incx+1,icy,icz)=d111i
 101     continue
         tmp=sr
         sr=sr*cnr+cr*snr+sr
         cr=cr*cnr-tmp*snr+cr
 100  continue
      call four21(d3,nnh,3,-1)

      return
      end

c==================================================

      subroutine four21(data,nn,ndim,isign)

      complex data(*),t(1048)
      integer nn(3)

      if(ndim.ne.3 .or. nn(3).gt.1024)stop 'four21 error'

      n2=nn(1)*nn(2)
      n3=n2*nn(3)
      do 1 i=1,n3,n2
 1    call fourn(data(i),nn,2,isign)

      do 20 i=0,n2-1
         nj=-i
         do 21 j=1,nn(3)
            nj=nj+n2
            t(j)=data(nj)
 21      continue
c         call four1(t,nn(3),isign)
         call fourn(t,nn(3),1,isign)
         nj=-i
         do 22 j=1,nn(3)
            nj=nj+n2
            data(nj)=t(j)
 22      continue
 20   continue
      return
      end

c=================================================

      SUBROUTINE FOURN(DATA,NN,NDIM,ISIGN)
c     DOUBLE PRECISION WR,WI,WPR,WTEMP,THETA
      DIMENSION NN(NDIM),DATA(*)
      NTOT=1
      DO 11 IDIM=1,NDIM
        NTOT=NTOT*NN(IDIM)
   11 CONTINUE
      NPREV=1
      DO 18 IDIM=1,NDIM
        N=NN(IDIM)
        NREM=NTOT/(N*NPREV)
        IP1=2*NPREV
        IP2=IP1*N
        IP3=IP2*NREM
        I2REV=1
        DO 14 I2=1,IP2,IP1
          IF (I2.LT.I2REV) THEN
            DO 13 I1=I2,I2+IP1-2,2
cdir$ IVDEP
              DO 12 I3=I1,IP3,IP2
                I3REV=I2REV+I3-I2
                TEMPR=DATA(I3)
                TEMPI=DATA(I3+1)
                DATA(I3)=DATA(I3REV)
                DATA(I3+1)=DATA(I3REV+1)
                DATA(I3REV)=TEMPR
                DATA(I3REV+1)=TEMPI
   12         CONTINUE
   13       CONTINUE
          ENDIF
          IBIT=IP2/2
    1     IF ((IBIT.GE.IP1) .AND. (I2REV.GT.IBIT)) THEN
            I2REV=I2REV-IBIT
            IBIT=IBIT/2
            GO TO 1
          ENDIF
        I2REV=I2REV+IBIT
   14   CONTINUE
        IFP1=IP1
    2   IF (IFP1.LT.IP2) THEN
          IFP2=2*IFP1
          THETA=ISIGN*6.28318530717959D0/(IFP2/IP1)
          WPR=-2.D0*SIN(0.5D0*THETA)**2
          WPI=SIN(THETA)
          WR=1.D0
          WI=0.D0
          DO 17 I3=1,IFP1,IP1
            DO 16 I1=I3,I3+IP1-2,2
cdir$ IVDEP
              DO 15 I2=I1,IP3,IFP2
                K1=I2
                K2=K1+IFP1
                TEMPR=WR*DATA(K2)-WI*DATA(K2+1)
                TEMPI=WR*DATA(K2+1)+WI*DATA(K2)
                DATA(K2)=DATA(K1)-TEMPR
                DATA(K2+1)=DATA(K1+1)-TEMPI
                DATA(K1)=DATA(K1)+TEMPR
                DATA(K1+1)=DATA(K1+1)+TEMPI
   15         CONTINUE
   16       CONTINUE
            WTEMP=WR
            WR=WR*WPR-WI*WPI+WR
            WI=WI*WPR+WTEMP*WPI+WI
   17     CONTINUE
          IFP1=IFP2
          GO TO 2
        ENDIF
        NPREV=N*NPREV
   18 CONTINUE
      RETURN
      END

c==================================================
c The Numerical Recipes routines are actually MINIMIZATION routines and
c so, to maximize nsphere, we need to minimize the corresponding
c negative function.
c Instead of minimizing the mass we are minimizing the delta mass
c since this is at early times. Same thing, it won't matter.

      function FUNC(x)

      real x(3),r(3),rL,rL1
      integer L

      common/ln/L,nobj

      rL=float(L)
      rL1=rL-1.

      do 66 i=1,3
 66      r(i)=mod(x(i)+rL1,rL)+1.

      FUNC=-deltafft(r)
      return
      end
c------------------------
      SUBROUTINE readr(iu,nobj,r,ier)
c
      real*4 r(3,nobj)
      read(iu,iostat=ier) r
      RETURN
      end
