      PROGRAM pnsum
c
      parameter(Lftmax=256)
      common/idf/rfil,idf,mn
      common /spectrum/ispectrum,index,omegahh
      tpi=6.2831853
c
      open(55,file='cosmic.dat',status='old')
      read(55,*) irun,iseed1
      read(55,*) ndark
      read(55,*) box100,h100
      read(55,*) omega0,xlambda0
      read(55,*) omegab0,gamma
      read(55,*) ispectrum,index
      read(55,*) atime,sigma80
      read(55,*) lcool,zmet
      close(55)
c
      ipow=4
      L=8
      do while (L**3.ne.ndark)
         L=2**ipow
         ipow=ipow+1
         if (L.gt.Lftmax) then 
            write(*,*) 'ndark**(1/3) must be power-of-2'
            write(*,*) 'and ndark <=',Lftmax**3
            stop
         endif
      enddo
c
      if (L.gt.Lftmax) STOP 'L>Lftmax'
      box=box100/h100
      if (gamma.le.1e-4 .and. ispectrum.gt.1) then
         gamma=omega0*h100*exp(-omegab0*(1.+1./omega0))
         write(*,*) 'Using gamma = ',gamma
      end if
      omegahh=gamma*h100
c
      idf=2                                    !top-hat filter
      rfil=8./h100                             !filter radius
      mn=0                                     !zero'th moment of PS
c
      sig8=pnorm(rfil,box)
      write(*,*) 'Analytic sigma_8 (box units) =',sig8
      write(*,*) 'Setting this equal to unity then:'
      Lft=Lftmax
      nwave=Lft/2
      rLbox=float(Lftmax)/float(L)*box
      res=sqrt(rsum(nwave,rLbox,Lft))
      write(*,*) 'Measured sigma_8 in box of size ',Lftmax,
     &     ' units (',rLbox*h100,'/h Mpc) =',res/sig8
      Lft=L
      nwave=Lft/2
      rLbox=box
      res=sqrt(rsum(nwave,rLbox,Lft))
      write(*,*) 'Measured sigma_8 in box of size ',L,
     &     ' units (',rLbox*h100,'/h Mpc) =',res/sig8
c
      end
c
      real function rsum(nwave,rLbox,Lft)
      parameter(Lftmax=256)
      common/idf/rfil,idf,mn
c
      tpi=6.2831853
      rLrLb=rLbox/float(Lft)
      tpiL=tpi/rLbox
      res=0.
      do 100 iz=0,nwave
         do 100 iy=0,iz
            do 100 ix=0,iy
               rk=tpiL*sqrt(float(ix**2+iy**2+iz**2))
               ifac=ialp(ix)*ialp(iy)*ialp(iz)*igam(ix,iy,iz)
               res=res+rmultpow(rk,rLrLb)*ifac
 100        continue
c  normalise by 1/rLbox^3 to get mean square deviations to compare with
c  pnorm.  Would be 1/rLbox^6 to get answer in units in which Fourier
c  transform of power spectrum takes a simple form (see Theory book
c  pages 26-7
      rsum=(1./rLbox)**3*res
c
      return
      end
c
      real function rmultpow(rk,rLrLb)
      parameter(pi=3.141592,rke=0.8*pi,Ne=16)
      real rk,x,rfil
      real powtem,powr
      common/idf/rfil,idf,mn
      Wth(x)=3.0*(sin(x)-x*cos(x))/(x**3)
c     
      powtem=pow(rk)*exp(-2.*(rk*rLrLb/rke)**Ne)
c     
      rmultpow=0.
      if(rk.gt.0.)then
         powr=powtem*(rk**mn)
         if(idf.eq.1)then
            rmultpow=powr*exp(-rk*rk*rfil*rfil)
         else if(idf.eq.2)then
            rmultpow=powr*(Wth(rfil*rk)**2)
         else if(idf.eq.3)then
            rmultpow=powr*Wth(rfil*rk)
         end if
      end if
      return
      end
c
      function ialp(i1)
      ialp=2
      if(i1.eq.0)ialp=1
      return
      end
c
      function igam(i1,i2,i3)
      igam=6
      if(i1.eq.i2 .or. i2.eq.i3 .or. i3.eq.i1)igam=3
      if(i1.eq.i2 .and. i2.eq.i3)igam=1
      return
      end
c-------------------------------------------------------------
      function pow(rk)
c  pow evaluates the power spectrum at wavenumber rk.
c  N.B. rk has units of 2pi/Mpc.
c
      common /spectrum/ispec,ind,omegahh
c
c  Primordial Harrison-Zel'dovich power spectrum.
c  Lauro uses this normalisation for some reason.
c      pow=69.56*q/(omegahh*omegahh*omegahh*bbias*bbias)
c
      if (rk.le.0.0) then
        pow=0.0
        RETURN
      end if
c
      if (ispec.eq.1) then
c  Power Law
         q=rk
         pow=q**ind
      else if (ispec.eq.2) then
C  CDM
         q=rk/omegahh
c  Transfer function for cold dark matter (from DEFW).
c  I have plotted this one up and it has a higher maximum and a higher
c  normalisation at large k than the others, so don't use it.
c         a1=1.7*q
c         a2=4.327*q
c         a3=1.0*q
c         t=1.0/(1.0+a1+a2*sqrt(a2)+a3*a3)
c  Transfer function for cold dark matter (from BBKS).
c         a1=2.34*q
c         a2=3.89*q
c         a3=16.1*q
c         a4=5.46*q
c         a5=6.71*q
c         t=1.0+a2+a3*a3+a4*a4*a4+a5*a5*a5*a5
c         t=log(1.0+a1)/a1/sqrt(sqrt(t))
c  Transfer function for cold dark matter (from Couchman).
         q=0.25*q
         a1=23.1*q
         a2=11.4*q
         a3=6.48*q
         t=(1.+(a1+a2**1.5+a3**2)**1.25)**(-0.8)
         pow=q*t**2
c         pow=t**2/q
      else if (ispec.eq.3) then
c  HDM
         q=rk/omegahh
c  Transfer function for hot dark matter (from BBKS).
         a1=2.6*q
         a2=1.6*q
         a3=4.0*q
         a4=0.92*q
         t=exp(-0.16*a1-0.5*a1*a1)/(1.0+a2+a3*sqrt(a3)+a4*a4)
         pow=q*t**2
      end if
c
      RETURN
      end
c--------------------------------------------------------------
      function pnorm(r,box)
c
c  Measured value of sigma within sphere of radius r Mpc
c
      parameter(tpi=6.28318530717959,pi=tpi/2)
      common /tophat/rth
      external rint
c
      rth=r
      CALL qromb(rint,0.,100./rth,sig2)
      sig2=4.*pi/tpi**3*sig2
      pnorm=sqrt(sig2)
c
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      function rint(x)
c
      common /tophat/rth
c
      rint=w(rth*x)*pow(x)*x**2
c
      RETURN
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      function w(x)
c
      if (x.le.1e-2) then
         w=1.-0.2*x**2
      else
         w=9.*(sin(x)-x*cos(x))**2/x**6
      end if
c
      RETURN
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccc
