      SUBROUTINE ahtime(t,omega0,xlambda0,a,h)
c
c  Evaluates the expansion factor a (=1 at t=1) and Hubble parameter
c  h (in units of 1/t0) at a given time t (in units of t0) and for
c  cosmological parameters omega0 and lambda0 (evaluated at t=1).
c
      save omega0old,xlambda0old,aold,h0
      data omega0old,xlambda0old/-1.,-1./
c
c  if cosmological parameters have changed then initialize things
      if (omega0old.ne.omega0 .or. xlambda0old.ne.xlambda0) then
         h0=aintegral(0.,1.,omega0,xlambda0)
         aold=t**(2./3.)
         omega0old=omega0
         xlambda0old=xlambda0
      end if
c
c  hunt for correct value of a using Newton-Raphson
      h0t=h0*t
      a=aold
      f=aintegral(0.,a,omega0,xlambda0)-h0t
      do while (abs(f).gt.1e-4*h0t)
         fprime=1/sqrt(1.+omega0*(1./a-1.)+xlambda0*(a**2-1.))
         a=a-f/fprime
         if (a.le.0. .or. a.gt.10.) STOP 'ahtime: not converging'
         f=aintegral(0.,a,omega0,xlambda0)-h0t
      end do
c
c  evaluate h and save value of a
      h=h0*sqrt(1.+omega0*(1./a-1.)+xlambda0*(a**2-1.))/a
      aold=a
c
      RETURN
      end
c-------------------------------------------------------------------
      real FUNCTION aintegral(a1,a2,omega,xlambda)
c
c  Does open-ended romberg integration of the a integral from a1 to a2
c  Based on a Numerical Recipes routine
c
      parameter (eps=1.e-6, jmax=14, jmaxp=jmax+1, k=5, km=k-1)
      real h(jmaxp),s(jmaxp)
      func(a,om,xla)=1/sqrt(1.+om*(1./a-1.)+xla*(a**2-1.))
c
      arange=a2-a1
      h(1)=1.
      do j=1,jmax
c  midpoint evaluation
	if (j.eq.1) then
           x=0.5*(a1+a2)
           s(j)=arange*func(x,omega,xlambda)
           it=1
	else
           tnm=it
           del=arange/(3.*tnm)
           ddel=2.*del
           x=a1+0.5*del
           sum=0.
           do jt=1,it
              sum=sum+func(x,omega,xlambda)
              x=x+ddel
              sum=sum+func(x,omega,xlambda)
              x=x+del
           end do
           s(j)=(s(j-1)+arange*sum/tnm)/3.
           it=3*it
	end if
c  extrapolate to zero step size
        if (j.ge.k) then
          CALL polint(h(j-km),s(j-km),k,0.,ss,dss)
          if (abs(dss).le.eps*abs(ss)) then
             aintegral=ss
             RETURN
          end if
        endif
        s(j+1)=s(j)
        h(j+1)=h(j)/9.
      end do
      STOP 'aintegral: too many steps'
c
      end
c--------------------------------------------------------
      SUBROUTINE polint(xa,ya,n,x,y,dy)
c  polynomial interpolation
      integer n,nmax
      real dy,x,y,xa(n),ya(n)
      parameter (nmax=10)
      integer i,m,ns
      real den,dif,dift,ho,hp,w,c(nmax),d(nmax)
c
      ns=1
      dif=abs(x-xa(1))
      do 11 i=1,n
        dift=abs(x-xa(i))
        if (dift.lt.dif) then
          ns=i
          dif=dift
        endif
        c(i)=ya(i)
        d(i)=ya(i)
11    continue
      y=ya(ns)
      ns=ns-1
      do 13 m=1,n-1
        do 12 i=1,n-m
          ho=xa(i)-x
          hp=xa(i+m)-x
          w=c(i+1)-d(i)
          den=ho-hp
          if(den.eq.0.)PAUSE 'failure in polint'
          den=w/den
          d(i)=hp*den
          c(i)=ho*den
12      continue
        if (2*ns.lt.n-m)then
          dy=c(ns+1)
        else
          dy=d(ns)
          ns=ns-1
        endif
        y=y+dy
13    continue
      RETURN
      end
