      program createtop
c
c     sets up an initial conditions file for a top hat.
c     does either both -DNOEXPAND -DISOLATED, or neither.
c
      include 'psize.inc'
#include "rvarrays.inc"
      include 'pinfo.inc'
      include 'itype.inc'
      include 'iodefs.inc'
      include 'units.inc'
c
c  the following radius gives a volume of 0.5
      parameter(radius=0.4923725,radius2=radius**2)
c  minimum temperatures in Kelvin for gas inside and outside sphere
      parameter(emininK=1.,eminoutK=1.)
c  output file name
      character outfile*10
      parameter(outfile='d1000.0000')
c  set particle number and check to make sure Nmax is big enough
      parameter(Nlin=16)
#ifdef NOSPH
      if (Nlin**3.gt.Nmax) STOP 'createtop: Nlin**3 > Nmax'
#else
      if (Nlin**3.gt.Nmaxgas) STOP 'createtop: Nlin**3 > Nmaxgas'
#endif
c
#ifdef NOEXPAND
#ifndef ISOLATED
c  don't allow NOEXPAND flag by itself
      STOP 'createtop: NOEXPAND requires ISOLATED'
#endif
#endif
c
      print*,'Creating a top-hat datafile.'
      print*,'Should collapse to a singularity at t=1 (if no softening)'
c
#ifdef NODARK
      Ndark=0
      print*,'Gas only'
#else
      Ndark=Nlin**3
#endif
#ifdef NOSPH
      Ngas=0
      print*,'Dark matter only'
#else
      Ngas=Nlin**3
#endif
      Nobj=Ndark+Ngas
c
#ifdef ISOLATED
      time=0.
      atime=1.
      htime=0.
#else
      box100=1.
      h100=1.
      omega0=1.
      xlambda0=0.
      time=0.01
      atime=time**(2./3.)
      htime=2./(3.*time)
#endif
      CALL inunit
#ifdef ISOLATED
      radreal=radius*lunit
c      print*,'radreal=',radreal
c  total mass required for sphere to collapse at t=1.
      rmunit=pi**2*(radreal/munit)*(radreal/tunit)**2/(8.*Msun*G)
c  mass per gas+dm particle in sphere (=1/2 unit box)
      rmunit=2.*rmunit/float(Nlin**3)
      rmnorm=9.*float(Ndark)+float(Ngas)
      rmdark=9.*float(Ndark)/rmnorm * rmunit
      rmgas=float(Ngas)/rmnorm * rmunit
#else
      rmdark=9.
      rmgas=1.
c  linear overdensity at maximum collapse for expanding box
      deltalin=0.6*(1.5*pi)**(2./3.)*atime
#endif
      eminin=emininK/(Kunit*atime**2)
      eminout=eminoutK/(Kunit*atime**2)
c
c  dump times
      tout1(1)=0.68
      tout1(2)=0.84
      tout1(3)=0.92
      tout1(4)=0.96
      tout1(5)=0.98
      tout1(6)=0.99
      tout1(7)=0.995
      tout1(8)=0.9975
      tout1(9)=0.99875
      tout1(10)=1.0
      tout1(11)=1.00125
      tout1(12)=1.0025
      tout1(13)=1.005
      tout1(14)=1.01
      tout1(15)=1.02
      tout1(16)=1.04
      tout1(17)=1.08
      tout1(18)=1.16
      tout1(19)=1.32
      tout1(20)=2.
      tout1(21)=0.
c
c  positions, velocities
      rfac=1./float(Nlin)
      roffset=-0.5-0.5*rfac
#ifdef ISOLATED
      dror=0.
#else
      dror=-deltalin/3.
#endif
      iobj=0
#ifndef NOSPH
      do ix=1,Nlin
         rx=roffset+ix*rfac
         do iy=1,Nlin
            ry=roffset+iy*rfac
            do iz=1,Nlin
               rz=roffset+iz*rfac
               rsq=rx**2+ry**2+rz**2
               if (rsq.gt.radius2) then
c outside sphere
                  continue
#ifndef ISOLATED
                  iobj=iobj+1
                  rm(iobj)=rmgas
c  reset particle type to collisionless
                  itype(iobj)=itstar
                  r(1,iobj)=0.5+rx
                  r(2,iobj)=0.5+ry
                  r(3,iobj)=0.5+rz
                  v(1,iobj)=0.
                  v(2,iobj)=0.
                  v(3,iobj)=0.
                  e(iobj)=eminout
                  h(iobj)=rfac
                  dn(iobj)=1.
#endif
               else
c  inside sphere
                  iobj=iobj+1
                  rm(iobj)=rmgas
                  itype(iobj)=1
                  drx=dror*rx
                  dry=dror*ry
                  drz=dror*rz
#ifdef ISOLATED
                  r(1,iobj)=rx+drx
                  r(2,iobj)=ry+dry
                  r(3,iobj)=rz+drz
#else
                  r(1,iobj)=0.5+rx+drx
                  r(2,iobj)=0.5+ry+dry
                  r(3,iobj)=0.5+rz+drz
#endif
                  v(1,iobj)=htime*drx
                  v(2,iobj)=htime*dry
                  v(3,iobj)=htime*drz
                  e(iobj)=eminin
                  h(iobj)=rfac*(1.-dror)
#ifdef ISOLATED
                  dn(iobj)=Ngas*rmgas
#else
                  dn(iobj)=1.+deltalin
#endif
               end if
            end do
         end do
      end do
#endif
c  reset ngas to actual number
      ngas=iobj
#ifndef NODARK
      do ix=1,Nlin
         rx=roffset+ix*rfac
         do iy=1,Nlin
            ry=roffset+iy*rfac
            do iz=1,Nlin
               rz=roffset+iz*rfac
               rsq=rx**2+ry**2+rz**2
               if (rsq.gt.radius2) then
c outside sphere
                  continue
#ifndef ISOLATED
                  iobj=iobj+1
                  rm(iobj)=rmdark
                  itype(iobj)=0
                  r(1,iobj)=0.5+rx
                  r(2,iobj)=0.5+ry
                  r(3,iobj)=0.5+rz
                  v(1,iobj)=0.
                  v(2,iobj)=0.
                  v(3,iobj)=0.
#endif
               else
c  inside sphere
                  iobj=iobj+1
                  rm(iobj)=rmdark
                  itype(iobj)=0
                  drx=dror*rx
                  dry=dror*ry
                  drz=dror*rz
#ifdef ISOLATED
                  r(1,iobj)=rx+drx
                  r(2,iobj)=ry+dry
                  r(3,iobj)=rz+drz
#else
                  r(1,iobj)=0.5+rx+drx
                  r(2,iobj)=0.5+ry+dry
                  r(3,iobj)=0.5+rz+drz
#endif
                  v(1,iobj)=htime*drx
                  v(2,iobj)=htime*dry
                  v(3,iobj)=htime*drz
               end if
            end do
         end do
      end do
#endif
c
c  reset nobj and ndark to actual numbers
      nobj=iobj
      ndark=nobj-ngas
c
      irun=1000
      intl=0
      nlmx=6
      perr=2.0
      dtnorm=1.
      sft0=0.1*rfac
      zmet0=0.5
      lcool=0
      itime=0
      CALL inunit
c
      datafile=outfile
      CALL dumpdata
c
      end
c-------------------------------------------------------------------
      FUNCTION RAN3(IDUM)
      SAVE
      PARAMETER (MBIG=1000000000,MSEED=161803398,MZ=0,FAC=1.E-9)
      DIMENSION MA(55)
      DATA IFF /0/
      IF(IDUM.LT.0.OR.IFF.EQ.0)THEN
        IFF=1
        MJ=MSEED-IABS(IDUM)
        MJ=MOD(MJ,MBIG)
        MA(55)=MJ
        MK=1
        DO 11 I=1,54
          II=MOD(21*I,55)
          MA(II)=MK
          MK=MJ-MK
          IF(MK.LT.MZ)MK=MK+MBIG
          MJ=MA(II)
11      CONTINUE
        DO 13 K=1,4
          DO 12 I=1,55
            MA(I)=MA(I)-MA(1+MOD(I+30,55))
            IF(MA(I).LT.MZ)MA(I)=MA(I)+MBIG
12        CONTINUE
13      CONTINUE
        INEXT=0
        INEXTP=31
        IDUM=1
      ENDIF
      INEXT=INEXT+1
      IF(INEXT.EQ.56)INEXT=1
      INEXTP=INEXTP+1
      IF(INEXTP.EQ.56)INEXTP=1
      MJ=MA(INEXT)-MA(INEXTP)
      IF(MJ.LT.MZ)MJ=MJ+MBIG
      MA(INEXT)=MJ
      RAN3=MJ*FAC
      RETURN
      END
