      SUBROUTINE accel
c
c  Evaluates the accelerations and the timestep constraint
c
      include 'psize.inc'
#include "rvarrays.inc"
      include 'pinfo.inc'
      include 'itype.inc'
      include 'units.inc'
#ifdef QOVERP
      include 'iodefs.inc'
#endif
#ifdef DUMPHVAR
      include 'iodefs.inc'
#endif
c
c  dummy array for densities
      real*4 dn0(Nmaxgas)

c  initialise timestep constraint and acceleration array
      dtv=1.e15
      do i=1,Nobj
         a(1,i)=0.
         a(2,i)=0.
         a(3,i)=0.
      end do
#ifndef NOSPH
      do i=1,Ngas
         de(i)=0.
#ifdef ENTROPY
         ds(i)=0.
#endif
      enddo
#endif

c  softening (used both for gravity and SPH)
      soft=softc(atime,sft0,sftmin,sftmax)

c  Gravity acceleration
#ifndef NOGRAV
      U=0.
      rmnorm=rmnorm0/atime**3
      CALL grav(Nobj,itype,rm,r,a,U,soft,rmnorm,perr,intl,nlmx)
      U=U*rmnorm0/atime
#endif

c  SPH acceleration
#ifndef NOSPH
      CALL sph
#endif

c  acceleration timestep
c  Note that soft2 is stored in pinfo.inc for possible use in lostp.
c  Note also that soft can be changed in grav (max 0.3) so don't move
c  this line up above that call.
      soft2=(soft/2.34)**2
      asqmax=1.e-10
      do i=1,Nobj
	asq=a(1,i)**2+a(2,i)**2+a(3,i)**2
	asqmax=max(asq,asqmax)
      enddo	
      dta=(soft2/asqmax)**0.25

c  Overall timestep control
#ifdef RELAX
c  Relaxation: this assumes SPH relaxation
c  note that it is a long time since this was looked at
c  and the dtv below may be far from optimal
      vsqmax=0.
      dnmax=0.
      dnmin=1.e30
      do i=1,Ngas
        dnmax=max(dnmax,dn(i))
        dnmin=min(dnmin,dn(i))
	vsq=v(1,i)**2+v(2,i)**2+v(3,i)**2
	vsqmax=max(vsq,vsqmax)
      enddo	
      print*,'density min, max = ',dnmin,dnmax
      dtv=sqrt(min(soft2,((dnmax-dnmin)/float(ngas))**0.666)/vsqmax)
      print*,'vmax=',sqrt(vsqmax),' dtv=',dtv
      dta=1./asqmax**0.25
      dtime=min(0.25*dta,0.4*dtv)*dtnorm
      print*, 'dtime, dta, dtv: ',dtime,0.25*dta,0.4*dtv
#else
#ifdef NOEXPAND
      dtime=min(0.25*dta,0.4*dtv,1.)*dtnorm
      print*, 'dtime, dta, dtv: ',dtime,0.25*dta,0.4*dtv
#else
      dtime=min(0.25*dta,0.4*dtv,0.0625/htime)*dtnorm
      print*, 'dtime, dta, dtv, dth: ',
     &     dtime,0.25*dta,0.4*dtv,0.0625/htime
#endif
#endif

#ifndef NOEXPAND
c  Hubble expansion
      do i=1,Nobj
#ifdef RELAX
c  this reduces the velocities by 0.5*dtnorm on each step
         a(1,i)=a(1,i)-0.5*dtnorm/dtime*v(1,i)
         a(2,i)=a(2,i)-0.5*dtnorm/dtime*v(2,i)
         a(3,i)=a(3,i)-0.5*dtnorm/dtime*v(3,i)
#else
         a(1,i)=a(1,i)-2.*htime*v(1,i)
         a(2,i)=a(2,i)-2.*htime*v(2,i)
         a(3,i)=a(3,i)-2.*htime*v(3,i)
#endif
      enddo
#ifndef NOSPH
#ifdef ENTROPY
c  The entropy is unaltered by a uniform expansion in physical co-ordinates,
c     but declines as atime**4 in code units.  We could define code entropy
c     to be in physical units but then we would have to pass the expansion
c     factor into sph.
      do i=1,Ngas
#ifdef RELAX
         ds(i)=0.
#else
         ds(i)=ds(i)-4.*htime*s(i)
#endif
      end do
#else  
c  If the gas forces are calculated using comoving velocities then use
c     -4*htime here.
c  An alternative is to convert to real velocities
c     for the hydrodynamics in which case we only need -2*time for the
c     use of comoving units (the expansion of the gas in real co-ordinates
c     will give us the other -2*htime)
      do i=1,Ngas
         de(i)=de(i)-4.*htime*e(i)
      end do
#endif
#endif
#endif

c  Diagnostic code
#ifdef QOVERP
      datafile='d1003.9999'
      CALL dumpdata
      STOP
#endif
#ifdef DUMPHVAR
      datafile='d1003.9998'
      CALL dumpdata
      STOP
#endif

      RETURN
      end
c
c *********************************************************************
c
      real function softc(a,sft0,sftmin,sftmax)
c
c  Evaluates the softening length
c
      softc=min(sftmax,max(sftmin,sft0/a))
c
      RETURN
      end
