      SUBROUTINE cool(den,epp,dt,zmet)
c  Does the cooling function in integrated form.
c
      INCLUDE 'pinfo.inc'
      INCLUDE 'units.inc'
c
c  ZMETEVOL is treated like ZMET in this subroutine
#ifdef ZMETEVOL
#define ZMET
#endif
c     
c  nlowt is the number of bins between 10^4K and 10^5K
c  nhight is the number of bins for each decade above 10^5K
c  nctab is the total number of bins beteen 10^4K and 10^10K
c  would suggest nlowt=101,nhight=10
c
      parameter(nlowt=101,nhight=10,nctab=nlowt+5*nhight)
#ifdef ZMET
      parameter(nzmet=100)
#else 
      parameter(nzmet=0)
#endif
      real ctab0(nctab,nzmet+1),ctab(nctab,nzmet+1)
      real ttab(nctab)
      logical firstcall
c
      save firstcall
      save told,it0,ic0,ttab,ctab0,ctab
      data firstcall/.true./
      data told,it0,ic0/-1.,-1,-1/
c
c  On the first call tabulate the cooling table
      if (firstcall) then
         firstcall=.false.
         CALL createcool(nctab,nlowt,nzmet,ctab0)
      end if
c
c  check for table update
      if (time.ne.told) then
         told=time
         ttab(1)=1.e4/(atime**2*Kunit)
         do i=2,nctab
            if (i.le.nlowt) then
               ttab(i)=ttab(i-1)*10.**(1./float(nlowt-1))
            else
               ttab(i)=ttab(i-1)*10.**(5./float(nctab-nlowt))
            endif
         enddo
         at3=atime**3
         do ii=1,nzmet+1
            ctab(1,ii)=0.
            do i=2,nctab
               ctab(i,ii)=ctab0(i,ii)*at3
            end do
         enddo
      end if
c     
c  find position in temperature table
      if (epp.le.ttab(1)) RETURN
      CALL hunt(ttab,nctab,epp,it0)
c
c  new value of integral
      if (it0.lt.1 .or. it0.gt.nctab) 
     &     STOP 'cool: something wrong in temperature hunt'
      if (it0.eq.nctab) then 
        print*,'max particle temperature exceeds 1e10K'
c if particle is very hot, reset it to 10^4K and print warning.
c See documentation
        epp=ttab(1)
        RETURN
      endif
c
c  do metallicity sanity checks (only for ZMET - zmet0 should be
c  checked at startup). We halt if zmet>1 because this is outside
c  the bounds of the cooling tables.
#ifdef ZMET
      if (zmet.lt.0.) STOP 'cool: negative metallicity'
      if (zmet.gt.1.) STOP 'cool: zmet exceeds 1'
#endif
      it1=it0+1
      nz=(zmet*nzmet)+1
      cint=(ctab(it1,nz)*(epp-ttab(it0))+ctab(it0,nz)*(ttab(it1)-epp))
     &     /(ttab(it1)-ttab(it0))-den*dt
c
c  find position in integral table
      if (cint.lt.0.) then
         epp=ttab(1)
         RETURN
      end if
      CALL hunt(ctab(1,nz),nctab,cint,ic0)
c
c  hence find new temperature
      if (ic0.eq.nctab) STOP 'cooled particle temp. exceeds initial'
      ic1=ic0+1
      epp=(ttab(ic1)*(cint-ctab(ic0,nz))+ttab(ic0)*(ctab(ic1,nz)-cint))
     &     /(ctab(ic1,nz)-ctab(ic0,nz))
c      
      RETURN
      end
      
c  it0, it1 - positions in the temperature table
c  ic0, ic1 - positions in cooling table
c  told - previous value of time
c  epp - temperature
c  den - gas number density
c  at3 - 1/time-dependence of T/Lambda
c  ttab - temperatures in grid units
c  ctab - int(T/Lambda) in grid units
c  cint - value of cooling integral
c  dt - timestep
c--------------------------------------------------
      SUBROUTINE hunt(xx,n,x,jlo)
c  finds position of entry in ordered table
c  value returned is the lower of the two bounding indices
c  0 is returned for underflows and n for overflows
      dimension xx(n)
      logical ascnd
      ascnd=xx(n).gt.xx(1)
      if(jlo.le.0.or.jlo.gt.n)then
        jlo=0
        jhi=n+1
        GOTO 3
      end if
      inc=1
      if(x.ge.xx(jlo).eqv.ascnd)then
1       jhi=jlo+inc
        if(jhi.gt.n)then
          jhi=n+1
        else if(x.ge.xx(jhi).eqv.ascnd)then
          jlo=jhi
          inc=inc+inc
          go to 1
        end if
      else
        jhi=jlo
2       jlo=jhi-inc
        if(jlo.lt.1)then
          jlo=0
        else if(x.lt.xx(jlo).eqv.ascnd)then
          jhi=jlo
          inc=inc+inc
          GOTO 2
        end if
      end if
3     if(jhi-jlo.eq.1)RETURN
      jm=(jhi+jlo)/2
      if(x.gt.xx(jm).eqv.ascnd)then
        jlo=jm
      else
        jhi=jm
      end if
      GOTO 3
      end
