      SUBROUTINE createcool(nctab,nlowt,nzmet,ctab0)
c
c  Interpolates from the Sutherland & Dopita tables to any metallicity
c  This routine should only be called ONCE (since cooling rates are not
c  saved between calls and rescaling is done on every call)  
c
c  ZMETEVOL is treated like ZMET in this subroutine
#ifdef ZMETEVOL
#define ZMET
#endif
c
#ifndef ZMET
      include 'pinfo.inc'
#endif
      include 'units.inc'
      include 'suthdop.dat'
c
      real ctab0(nctab,nzmet+1)
      real temp(ntemp),lfind(ntemp),lamc
c
      logical firstcall
      save firstcall
      data firstcall/.true./
c
c  check to make sure not called more than once
      if (firstcall) then
         firstcall=.false.
      else
         STOP 'createcool called more than once'
      end if
c
c     conversion to Hydra units (we use (n_e+n_i)**2, not n_e*n_H)
c     n_en_H/n_t^2=0.52*0.44=0.229
      scale=0.229 
      do i=1,ntemp
         temp(i)=10.**(4.000+0.05*(i-1))
         do ii=1,nz  
            lambda(ii,i)=scale*10.**lambda(ii,i)
         enddo       
      enddo       
c                 
      do ii=0,nzmet   
#ifdef ZMET
        zmetc=float(ii)/float(nzmet)
#else
        zmetc = zmet0
#endif
        ic=0
        do i=1,nz
           if (zmet(i).le.zmetc) ic=i
        enddo
#ifdef ZMET
        if (ic.eq.0) STOP 'createcool: zmet out of range'
#else
        if (ic.eq.0) STOP 'createcool: zmet0 out of range'
#endif
        if (ic.eq.nz) then
           ic=nz-1
           if (zmet0.ge.zmet(nz)+1e-6) 
     &          print*,'createltab: warning, extrapolating zmet'
        end if
        do i=1,ntemp
           lfind(i)=lambda(ic,i)+(lambda(ic+1,i)-lambda(ic,i))*
     &          (zmetc-zmet(ic))/(zmet(ic+1)-zmet(ic))
        enddo       
        residual=(sqrt(10.**8.5)*5.2e-28)-lfind(ntemp)
        told=1.e4
        ctab0(1,ii+1)=0.
        ic=1
        do i=2,nctab
           if (i.le.nlowt) then
              tnew=told*10.**(1./float(nlowt-1))
           else
              tnew=told*10.**(5./float(nctab-nlowt))
           endif
           tc=(tnew+told)/2.
 1         if ((tc.gt.temp(ic+1)).and.((ic+1).lt.ntemp)) then
              ic=ic+1
              goto 1
           endif
           if (tc.le.temp(ntemp)) then
              lamc=lfind(ic)+(lfind(ic+1)-lfind(ic))*
     &             (tc-temp(ic))/(temp(ic+1)-temp(ic))
           else
c  bremsstrahlung approximation at high temperatures
              lamc=(sqrt(tc)*5.2e-28)-residual
           endif                 
           ctab0(i,ii+1)=ctab0(i-1,ii+1)+1.5*kb*(tnew-told)/lamc/cunit
           told=tnew
        enddo
      enddo
      RETURN
c     
      end         

