      SUBROUTINE grav(N,itype,rm,r,a,U,soft,rmnorm,perr,intl,nlmx)
c
c  Creates list of refinements and runs through it calling refforce
c  to evaluate the accelerations as it goes.
c
      include 'psize.inc'
      include 'refparam.inc'
c
c  softmax is the maximum allowable softening in grid units.
c  It should be between 0.3 for (ultra-accurate forces) and 0.6
      parameter(softmax=0.3)
c
      real r(3,N),a(3,N),rm(N)
      real rp(3,Npmax),ap(3,Npmax),rmp(Npmax)
      real rmin(3),rfftb(3)
      integer itype(N),itypep(Npmax)
      integer ll(Nmax),llp(Npmax)
      integer ihc(Lsmax3),itag(Lsmax3),nhc(Lsmax3)
c
c  equivalence some arrays to save space
      common /wrk/ work(Nwrk)
      equivalence(work,ll),(work(Nmax+1),ihc)
      equivalence(work(Nmax+Lsmax3+1),nhc),(work(Nmax+2*Lsmax3+1),llp)
c
c  set size of PM mesh (L)
c    --- this need only be done once but, as it is a trivial number
c        of operations that doesn't really matter
      L=Lset(N,Lmax)
c
c  ip = this refinement number; mp is total number of refinements
      ip=0
      mp=0
c
c  set boundary condition flag, ibc=1 implies periodic
#ifdef ISOLATED
      ibc=0
#else
      ibc=1
#endif
c
c  Find the extent of the data
#ifdef ISOLATED
      CALL extent(N,itype,r,rmin,rsize)
#else
      do j=1,3
         rmin(j)=0.
      end do
      rsize=1.
#endif

c  decide upon optimum chaining cell-length,
c    find scalings to PM and PP meshes
c    and initialise refinement information,
      CALL rfinit(N,ibc,L,perr,intl,rmin,rsize,
     &     Ls,rf,srf,rffta,rfftb,rsa,rsb,err,softm,Nlins,psf)
c
c  scale data onto PP mesh
c  --- this is done to make the provide an interface to the Hydra3.0
c      version of the code.  Hopefully, this scaling will not be
c      necessary in some future version of the code.
c      I include a just in case it is non-zero (eg SPH done first)
      U=U/rffta
      soft=soft*rffta
      if (soft.gt.softmax) then
#ifndef QUIET
         print*,'**Warning: soft>softmax, resetting**'
#endif
         soft=softmax
      end if
      rmnorm=rmnorm*rffta**3
      CALL rescaleup(N,r,a,rffta,rfftb)
c
c  below here there will be minimal commenting.  The only things that 
c  have been altered from Hydra3.0 are that the refinement parameter
c  passing is now explicit and that the dptl correction is now worked
c  out in refforce upon the first call.
c
c  calculate forces at top level
      cpugravin=cputime()
      CALL refforce(N,rm,r,a,itype,ll,L,Ls,ihc,nhc,itag,rf,srf,
     &  rsa,rsb,softm,ibc,U,err,intl,nlmx,soft,rmnorm,Nlins,psf,mp,ip)
      cpugravout=cputime()
#ifdef CPUTIME
      print*,'cputime top grav = ',cpugravout-cpugravin
#endif
c
c  create linked list of particles in refinements
      CALL clist(N,r,itype,ll,Ls,itag,rsa,rsb,Nlins,mp)
c
      ibc=0
      ip=1
      do while (ip.le.mp)
#ifdef VERBOSE
         print*,'  refinement ',ip
#ifdef CPUTIME
         cpugravref0=cputime()
#endif
#endif
         Np=NLins(1,ip)
         CALL load(N,rm,r,a,itype,ll,Np,rmp,rp,ap,itypep,
     &        Lp,Lsp,rf,srf,rsa,rsb,err,intl,softm,Nlins,psf,ip)
         CALL refforce(Np,rmp,rp,ap,itypep,
     &        llp,Lp,Lsp,ihc,nhc,itag,rf,srf,rsa,rsb,softm,ibc,U,
     &        err,intl,nlmx,soft,rmnorm,Nlins,psf,mp,ip)
         CALL uload(N,a,itype,ll,Np,rp,ap,itypep,Lsp,
     &        itag,rsa,rsb,intl,Nlins,psf,ip)
         ip=ip+1
#ifdef VERBOSE
#ifdef CPUTIME
         cpugravref=cputime()-cpugravref0
         print*,'   cputime ',cpugravref
#endif
#endif
      enddo
      cpurefout=cputime()
#ifdef CPUTIME
      print*,'cputime ref grav = ',cpurefout-cpugravout
#endif
c
c  rescale back to original data units
      U=U*rffta
      soft=soft/rffta
      rmnorm=rmnorm/rffta**3
      CALL rescaledown(N,r,a,rffta,rfftb)
c
      RETURN
      end
c-------------------------------------------------------------------
      integer FUNCTION Lset(nobj,Lmax)
c
c  sets optimal size of base-level FFT mesh
c  if most of the particles have itype=itnone, then should search over
c  itype below
c
      ilp=float(nobj)**(1./3.)
      ic=0
      do while (ilp.ne.0)
	ilp=ilp/2
        ic=ic+1
      enddo
      Lset=max(2**ic,16)
      if (Lset.gt.Lmax) then
#ifndef QUIET
         print*,'*** Warning Lmax is too small for N ***'
         print*,'    Would like Lmax = ',Lset
#endif
         Lset=Lmax
      endif
#ifdef VERBOSE
      print*,'Setting size of PM mesh to ',Lset
#endif
c
      RETURN
      end
