      SUBROUTINE photo(atime)
c
c     N.B. Parameters for the photoionizing background
c     are defined in photo.inc
c
      include 'photo.inc'
c
c     This subroutine calculates the steady-state temperature for
c     a gas at a range of densities and places the results in piTmin.
c     The ionization, heating and cooling rates are calculated for a
c     single value of the photo-ionizing background J21 at z=0, rj210.
c     The equilibrium state depends upon the ratio of the J21 to
c     the density and so the density needs to be scaled by 
c     rjratioi=rj21/rj210 before look-up in the table.

      real atime,redshift
      logical firstcall
      save firstcall
      data firstcall/.true./
c
c     tables for heating, cooling
c
      real hTab(nDen,nTem)
      real cTab(nDen,nTem)
      real el_abund(nDen,nTem)
      real ff(nDen, nTem)
      save hTab,cTab,el_abund,ff
      real working(nTem)
      
c     on firstcall calculate cooling and heating tables for J_21 at z=0

      if (firstcall) then
         firstcall = .false.

         rj210=rj21z(0.)
         rj21=rj210
         
         do j=1,nTem
            tArr(j) = 10**(tmin + (float(j-1)/float(nTem))*tRange)
            temperature = tArr(j)
            do i=1,nDen
               dnArr(i)=10**(dnmin + (float(i-1)/float(nDen))*dnRange)
               density = dnArr(i)
               CALL calcAbundances(density,temperature)
               el_abund(i,j) = abund(6)
               hTab(i,j) = Heating()/density               
               cTab(i,j) = Cooling(temperature)
               ff(i,j) = Bremsstrahlung(temperature)
            end do
         end do         
      endif 

      redshift=1./atime-1.
      rj21ratioi=rj210/rj21z(redshift)
      if (redshift.lt.reion) then 
         do i=1,nDen
            do j=1,nTem
               cc=Compton(tArr(j),el_abund(i,j),redshift)/dnArr(i)
               working(j) = hTab(i,j)-(cTab(i,j)+ff(i,j)+cc)
            end do
            CALL hunt(working,nTem,0.,ic0)
            if (ic0.eq.0) then
               piTmin(i)=tArr(1)
            else if (ic0.eq.nTem) then
               piTmin(i)=tArr(nTem)
            else
               ic1=ic0+1
               piTmin(i)=(tArr(ic0)*working(ic1)-
     &            tArr(ic1)*working(ic0))/(working(ic1)-working(ic0))
            end if
         end do
      else
         do i=1,nDen
            piTmin(i) = 0.
         end do
      endif

      RETURN
      END
      
C==================================================================

      real function rj21z(z)

      include 'photo.inc'

c     evolution from Haardt & Madau 1996

      parameter(rj21norm=0.22,zp=0.73,zmax=2.3,zsig=1.9)

      if (z.lt.reion) then 
         rj21z = rj21norm * (1+z)**zp * exp(-(z-zmax)**2/zsig)
      else
         rj21z = 0.
      endif
c
c  to prevent division by zero in inverse
      rj21z=max(1.e-20,rj21z)

      RETURN
      END

c=====================================================================
c
      real FUNCTION emin_ss(dnpp)
c
      include 'photo.inc'
c
      integer id0
      real*4 dnpp

      dnr=dnpp*dnhtot*rj21ratioi

      call hunt(dnArr,nDen,dnr,id0)

      if (id0.eq.0) then
         emin_ss=piTmin(1)
      else if (id0.eq.nDen) then
         emin_ss=0.
      else
         emin_ss = (piTmin(id0)+(piTmin(id0+1)-piTmin(id0))*
     &        (dnr-dnArr(id0))/(dnArr(id0+1)-dnArr(id0)))
      endif
c      
      return
      end
c
C==================================================================

      SUBROUTINE calcAbundances(density,temperature)
      
      include 'photo.inc'
      
c     flag to see if we''ve finished iterating
      logical finished
      
c     abw + abwold are working arrays. They contain, in order
c     H1, He1, He2, He3, e
c     where H1 == n_H1/n_H i.e. the ionisation fraction
      real abw(5),abwold(5),density_old
c     save abwold allowing faster convergence
      save abwold
      data density_old/-1./

      CALL calcIonRates(temperature)

c     if rj21 = 0 then collisional ionization only and don't
c     need to iterate
            
      if (rj21.le.0.) then         
         abund(1) = recH2/(colH1 + recH2)
         abund(2) = 1.-abund(1)
         
         if (colHe1.gt.1.e-10) then 
            abund(4)   = y / (1.0 + recHe2/colHe1 + colHe2/recHe3)
            abund(5) = abw(3)*colHe2/recHe3
            abund(3) = abw(3)*recHe2/colHe1
         else
            abund(3) = y
            abund(4) = 0.
            abund(5) = 0.
         endif
         
         abund(6) = abund(2) + abund(4) + 2.*abund(5)

      else
         
c     If density has changed a lot then assume everything is ionised,
c     otherwise use old values
         if (abs(density-density_old).gt.0.3*density) then
            abwold(1) = 0.
            abwold(2) = 0.
            abwold(3) = 0.
            abwold(4) = y
            abwold(5) = 1+2.*y
         end if
         density_old=density

         icount = 0
         finished = .false.
         abw(5) = abwold(5)

c     iterate to find solution
c
c     we have three continuity equations,
c
c     (1)   H1 + H2 = 1
c     (2)   He1 + He2 + He3 = y 
c     (3)   H2 + He2 + 2 x He3 = e 
c
         do while(finished.eqv..false.)
            finished = .true.
            icount = icount + 1
            if (icount.gt.maxiter-10) print*,'abw: ',abw
            if (icount.gt.maxiter) then
               print*,'photo: too many iterations'
               STOP
            end if

c     electron number density 

            ei   = 1./(abw(5)*density)
            abw(1) = recH2/(recH2+colH1+photoH1*ei)

c     continuity equation (1)

            H2 = 1.-abw(1)

            r23=recHe2/(photoHe1*ei+colHe1)
            r43=(photoHe2*ei+colHe2)/recHe3
            abw(3)   = y / (1.0 + r23 + r43)
c            ratio=(1.+r43)/r23
c            if (ratio.lt.1.0) then
c               abw(2) = y / (1.0 + ratio)
               abw(4) = abw(3) * r43
c            else
               abw(2) = abw(3) * r23
c               ratio=1./ratio
c               abw(4) = y / (1.0 + ratio)
c            end if
         
            abw(5) = H2 + abw(3) + 2.*abw(4)
         
c     check to see if change in abundances on this iteration are
c     within the tolerance parameter

            do ir = 1,5
               if (abs(abw(ir)-abwold(ir)).gt.
     &           max(1.e-20,tolerance*abwold(ir))) finished = .false.
            enddo

c     now, if we''re not finished, go around again

            if (finished.eqv..false.) then
               etmp = max((abw(5)+abwold(5))/2.,minion)
               do ir = 1,5
                  abwold(ir) = abw(ir)
               end do
               abw(5) = etmp
            endif
         enddo
         abund(1) = abw(1)
         abund(2) = 1.-abw(1)
         abund(3) = abw(2)
         abund(4) = abw(3)
         abund(5) = abw(4)
         abund(6) = abw(5)
      endif

      RETURN
      END

C==================================================================

      SUBROUTINE calcIonRates(T)

      include 'photo.inc'

      real T
      real*8 recHe2d,T5th,T5x,Ttmh,Tth
      
      T3 = T/1.e3
      T5 = T/1.e5
      T5th = T5**0.5
      T5x = (1.+T5th)**(-1.)
      T6 = T/1.e6
      Ttmh = T**(-1./2.)
      Tth = T**(1./2.)

c     From Theuns et al 1998
c     
c     Recombination
c
      recH2 = 6.30d-11*Ttmh*T3**(-0.2)/(1.+T6**(0.7))
      recHe2d = 1.9d-3*T**(-1.5)*exp(-4.7d5/T)*(1.+0.3*exp(-9.4d4/T))
      recHe2 = 1.5d-10*T**(-0.6353d0) + recHe2d
      recHe3 = 3.36d-10*Ttmh*T3**(-0.2)/(1.+T6**(0.7))
c
c     Collisional Ionization
c      
      colH1 = 1.17d-10*Tth*exp(-157809.1/T)*T5x
      colHe1 = 4.76d-11*Tth*exp(-285335.4/T)*T5x
      colHe2 = 1.14d-11*Tth*exp(-631515./T)*T5x
c
c     Photoionization
c
      photoH1 = 1.26e-11*rj21/(3.+alpha_j21)
      photoHe1 = 1.48e-11*rj21*(0.553**alpha_j21)
     &     *(1.66/(alpha_j21+2.05) + 0.66/(alpha_j21+3.05))
      photoHe2 = 3.34e-12*rj21*(0.259**alpha_j21)/(3+alpha_j21)

      RETURN
      END

C==================================================================

      real FUNCTION Heating()

      include 'photo.inc'

      a2 = 1./(2.+alpha_j21)
      a3 = 1./(3.+alpha_j21)

      heatH1 = 2.91e-22*rj21*a2*a3
      heatHe1 =5.84e-22*rj21*(0.553**alpha_j21)*((1.66/(alpha_j21+1.05))
     &     - (2.32/(alpha_j21+2.05)) + (0.66/(alpha_j21+3.05)))
      heatHe2 = 2.92e-22*rj21*(0.249**alpha_j21)*a2*a3

      heating = (abund(1) * heatH1
     &     + y * abund(3) * heatHe1
     &     + y * abund(4) * heatHe2)
      
      RETURN
      END

C==================================================================

      real FUNCTION Cooling(temperature)

      include 'photo.inc'
      real temperature

c     commonly used abundances

      H1 = abund(1)
      H2 = abund(2)
      He1 = abund(3)
      He2 = abund(4)
      He3 = abund(5)
      elecnum = abund(6)

c     commonly used values

      T = temperature
      Tth = sqrt(T)
      T3 = T/1.e3
      T5 = T/1.e5
      T5th = sqrt(T5)
      T5x = (1.+T5th)**(-1.)
      T6 = T/1.e6
      Ttmh = T**(-1./2.)

c     rates, again from Theuns
c
c     Collisional Excitation
c     
      tfac = Tth*T5x
      cColH1 = 2.54e-21*exp(-157809.1/T)*tfac
      cColHe1 = 1.88e-21*exp(-285335.4/T)*tfac
      cColHe2 = 9.90e-22*exp(-631515./T)*tfac
c
c     Recombination cooling
c
      tfac = Tth * T3**(-0.2) * (1.+T6**0.7)**(-1.) 
      cRecH2 = 8.70e-27 * tfac
      cRecHe2 = 1.55e-26 * T**(0.3647)
      cRecHe3 = 3.48e-26 * tfac
c     
c     Dielectronic cooling (whatever that is)
c     
      cDRecHe2 = 1.24e-13 * T**(-1.5) * exp(-470000./T) *
     &     (1.+ 0.3*exp(-94000/T))
c
c     Collisional Excitation cooling
c     
      cColExH1 = 7.5e-19 * exp(-118348/T)*T5x
      cColExHe2 = 5.54e-17 * T**(-0.397) * exp(-473638./T) * T5x
c     
c     Bremsstrahlung (gff is the gaunt fudge factor)
c     
      cBremgff = 1.1 + 0.34*exp((-1./3)*(5.5-log10(T))**2.)
      cBrem = 1.42e-27 * cBremgff * Tth

c     collisional Ionisation cooling

      collisionalIon = cColH1*H1 + cColHe1*He1 + cColHe2*He2

c     recombination cooling

      recombination = cRecH2*H2 + cRecHe2*He2 + cRecHe3*He3

c     dielectric recombination cooling (whatever that is?)

      dielectronic = cDRecHe2*He2

c     collisional excitation cooling

      collisionalExcite = cColExH1*H1 + cColExHe2*He2

c     bremsstrahlung

      bremsstrahlung = cBrem * (H2 + He1 + 4.*He2)

c     add them all together
      Cooling = (collisionalIon + recombination + dielectronic + 
     &     collisionalExcite + bremsstrahlung)*elecnum

      RETURN
      END

c==================================================================
      
      real FUNCTION Bremsstrahlung(temperature)

      include 'photo.inc'
      real temperature

      H2 = abund(2)
      He1 = abund(3)
      He2 = abund(4)

c     commonly used values

      T = temperature
      Tth = sqrt(T)

      cBremgff = 1.1 + 0.34*exp((-1./3)*(5.5-log10(T))**2.)
      cBrem = 1.42e-27 * cBremgff * Tth

      Bremsstrahlung = cBrem * (H2 + He1 + 4.*He2)

      return
      end
c==================================================================

      real FUNCTION Compton(temperature,eab,redshift)

      include 'photo.inc'

c     inverse compton cooling

      opz = 1+redshift

      if (temperature.gt.2.7*opz) then
         Compton = 5.406e-36*(temperature-2.7*opz)*(opz**4.)*eab
      else
         Compton = 0.
      endif

      RETURN
      END

