      SUBROUTINE shsph(N,Lg,ll,ihc,hmin,hppmin)
c
c  Calculates sph forces for all particles that have not yet been done
c    and that have h>hppmin
c
c  Array dn0 is needed to make the neighbour count scan-independent
c  when using -DWEIGHTED_NEIGHBOUR
c
#ifdef TRACE
      parameter (itest=1)
#endif
#ifndef SPH2
c  using SPH1 as SPH2 was not specified
#define SPH1
#endif
#ifdef DUMPHVAR
c  DUMPHAR is incompatible with NOHVAR
#undefine NOHVAR
#endif

      include 'psize.inc'
#include "rvarrays.inc"
      include 'pinfo.inc'
      include 'itype.inc'
#ifdef DECOUPLE
      include 'units.inc'
#endif
c
      integer ihc(Lg,Lg,Lg),ll(N)
c
c  accounting
      integer jnbox(-1:1,-1:1,-1:1)
      real radd(0:Lgmax+1),rxadd(-1:1),ryadd(-1:1),rzadd(-1:1)
      integer indx(0:Lgmax+1),inx(-1:1),iny(-1:1),inz(-1:1)
c
c  sph
      parameter(nsppmax=32768,nsph=32,sphni=1./(1.+nsph))
      parameter(hsearchfac=1.2)
      parameter(onethird=1./3.,twothirds=2./3.)
      parameter(onethirdi=-1./3.,tenninth=10./9.)
c  avisc and bvisc are magic coefficients from shock tube experiments
c  avisc=5./6. and bvisc=5./3. are equivalent to Monaghan,s alpha=1
c   and beta=2 (for particles with identical properties).
#ifdef NOAV
      parameter(avisc=0.,bvisc=0.)
#else
      parameter(avisc=1.,bvisc=2.)
#endif
#ifdef SPH2
      parameter(qoverp=1.)
      real pgrad(3)
#endif
      real rpp(3),vpp(3),app(3),dr(3),dvr(3),dvc(3)
      integer jlist(nsppmax)
#ifdef EXACTNSPH
      integer index(nsppmax)
#endif
      real dx(3,nsppmax)
      real w(nsppmax),dw(nsppmax),rdvr(nsppmax),radii2(nsppmax)
      real rdvc(nsppmax)
#ifdef BALSARA
      real rcv(3),dcvpp(3) 
#endif
#ifdef DECOUPLE
      logical decouplepp,decouple
#endif
c
c  kernel arrays and evaluation flag
      real wtab(0:ntp+1), dwtab(0:ntp+1)
      logical firstcall
      save firstcall
      save wtab,dwtab
      data firstcall/.true./
c
c  On the first call evaluate the smoothing kernel and its derivative
      if (firstcall) then
         firstcall=.false.
         CALL createkernel(ntp,wtab,dwtab)
      end if
c
c  initialise diagnostic counters
      nstot=0
c  the nspppmax below is not a typo---nsppmax is the max allowed nspp
      nspppmax=0
      nsppmin=nsppmax
      nspptot=0
#ifdef WEIGHTED_NEIGHBOUR
      wnsppmax=0.
      wnsppmin=1.e30
      wnspptot=0.
#endif
      fhvarmin=1e30
      fhvarmax=0.
      fhvartot=0.
#ifdef DECOUPLE
c  upper and lower limits for decoupling of hot and cold gas
      e1e4K=1.e4/Kunit/atime**2
      elow=1.2*e1e4K
      ehigh=10.*e1e4K
#endif
c
c  dtv is the velocity timestep control
      dtv2=dtv**2
c
c  wrapping
      radd(0)=-1.
      indx(0)=Lg
      do i=1,Lg
         radd(i)=0.
         indx(i)=i
      enddo
      radd(Lg+1)=1.
      indx(Lg+1)=1
c  loop over central boxes ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
c  this would be an ideal place to share out work on an SMP machine
      do ibz=1,Lg
         do iby=1,Lg
            do ibx=1,Lg
c
c  set up wrapping
      do i=-1,1
         rxadd(i)=radd(ibx+i)
         ryadd(i)=radd(iby+i)
         rzadd(i)=radd(ibz+i)
         inx(i)=indx(ibx+i)
         iny(i)=indx(iby+i)
         inz(i)=indx(ibz+i)
      end do

c  loop over neighbouring boxes setting up temporary linked lists 
      do nbz=-1,1
         do nby=-1,1
            do nbx=-1,1
               jnbox(nbx,nby,nbz)=ihc(inx(nbx),iny(nby),inz(nbz))
            enddo
         enddo
      enddo

c  Loop over particles in central box -----------------------------------
      i=jnbox(0,0,0)
      do while (i.gt.0)

       if (itype(i).eq.itgas) then
c  do the sph if required
#ifdef EXACTNSPH
c  search out to hsearchfac times the previous smoothing radius
        hpp=max(hmin,hsearchfac*h(i))
#else
        hpp=max(hmin,h(i))
#endif
        if (hpp.gt.hppmin) then
         itype(i)=itdone
         rad2max=4.*hpp**2
         nspp=0
         rpp(1)=r(1,i)
         rpp(2)=r(2,i)
         rpp(3)=r(3,i)
#ifdef DECOUPLE
         decouplepp=.false.
         if (epp.gt.ehigh) decouplepp=.true.
#endif

c  Find neighbouring particles within smoothing radius
c  Loop over neighbour boxes ============================================
         do nbz=-1,1
            r3i=rpp(3)-rzadd(nbz)
            do nby=-1,1
               r2i=rpp(2)-ryadd(nby)
               do nbx=-1,1
                  r1i=rpp(1)-rxadd(nbx)
c     loop over particles in box +++++++++++++++++++++++++++++++++++++++++
         j=jnbox(nbx,nby,nbz)
         do while (j.gt.0) 
            if (j.ne.i) then
               rad2=(r(1,j)-r1i)**2+(r(2,j)-r2i)**2+(r(3,j)-r3i)**2
#ifdef DECOUPLE
               decouple=.false.
               if (decouplepp .and. e(j).lt.elow) decouple=.true.
               if (rad2.lt.rad2max .and. .not.decouple) then
#else
               if (rad2.lt.rad2max) then
#endif
                  nspp=nspp+1
                  if (nspp.gt.nsppmax) STOP 'shsph: nspp > nsppmax'
#ifdef EXACTNSPH
                  index(nspp)=nspp
#endif
                  jlist(nspp)=j
                  radii2(nspp)=rad2
c  relative positions must be evaluated here because they depend upon 
c  the wrapping which can vary from particle to particle
                  dx(1,nspp)=r(1,j)-r1i
                  dx(2,nspp)=r(2,j)-r2i
                  dx(3,nspp)=r(3,j)-r3i
               endif
            endif
            j=ll(j)
         end do
c     finished box ++++++++++++++++++++++++++++++++++++++++++++++++++
               end do
            end do
         enddo
c     finished neighbour boxes ======================================

#ifdef EXACTNSPH
         if (nspp.gt.nsph) then
c  partial sort into particles closer and further than nsph'th one
            r2nsph=SELECT_INDEX(nsph,nspp,radii2,index)
            hpp=0.5*sqrt(r2nsph)
            nspp=nsph
         end if
#endif
            
c  Initialise SPH variables (wow there are a lot of these)
         hppi=1./hpp
         hpp2=hpp**2
         hpp3=hpp2*hpp
         hpp2i=hppi*hppi
         hpp3i=hpp2i*hppi
         hpp5i=hpp3i*hpp2i
         rad2torbin=(0.25*float(ntp))*hpp2i
#ifdef WEIGHTED_NEIGHBOUR
         wnspp=0.
         dnpp=dn0(i)
#else
         dnpp=dn(i)
#endif
         rmpp=rm(i)
         do ixyz=1,3
            vpp(ixyz)=v(ixyz,i)
            app(ixyz)=0.
         enddo
         epp=e(i)
         cspp=sqrt(tenninth*epp)
         depress = 0.
         deart = 0.
         dbarpp=rmpp*wtab(0)
#ifdef SPH2
         pbarpp=rmpp*epp*wtab(0)
         do k=1,3
            pgrad(k)=0.
         end do
#endif
         divppr=0.
         divppc=0.
#ifdef BALSARA
         dcvpp(1)=0.
         dcvpp(2)=0.
         dcvpp(3)=0.
#endif
         xdwsum=0.

c  First sph loop over neighbouring particles.
c  Accumulates density, velocity divergence, etc
c  Index for jlist and position arrays is isppindex
c  Index for other sph arrays is ispp
c  These may differ if the -DEXACTNSPH flag is defined
         do ispp=1,nspp
#ifdef EXACTNSPH
            isppindex=index(ispp)
#else
            isppindex=ispp
#endif
            j=jlist(isppindex)
c
#ifdef WEIGHTED_NEIGHBOUR
c  weighted number count 
c     (assumes that we know dn - a count based on h is unstable)
            wnspp=wnspp+2.*dnpp/(dnpp+dn0(j))
#endif
            rmj=rm(j)
c  separation and relative positions
            rad2=radii2(isppindex)
            dr(1)=dx(1,isppindex)
            dr(2)=dx(2,isppindex)
            dr(3)=dx(3,isppindex)
c  relative velocities in real and comoving co-ordinates
            dvr(1)=v(1,j)-vpp(1)+htime*dr(1)
            dvr(2)=v(2,j)-vpp(2)+htime*dr(2)
            dvr(3)=v(3,j)-vpp(3)+htime*dr(3)
            dvc(1)=v(1,j)-vpp(1)
            dvc(2)=v(2,j)-vpp(2)
            dvc(3)=v(3,j)-vpp(3)
            rdvr(ispp)=dr(1)*dvr(1)+dr(2)*dvr(2)+dr(3)*dvr(3)
            rdvc(ispp)=dr(1)*dvc(1)+dr(2)*dvc(2)+dr(3)*dvc(3)
#ifdef BALSARA
            rcv(1)=dr(2)*dv(3)-dr(3)*dv(2)
            rcv(2)=dr(3)*dv(1)-dr(1)*dv(3)
            rcv(3)=dr(1)*dv(2)-dr(2)*dv(1)
#endif
c  evaluate dtv here
c  include a dependence on e simply to satisy referees who are
c  fixated on the Courant condition
            v2=dvr(1)*dvr(1)+dvr(2)*dvr(2)+dvr(3)*dvr(3)+1.e-30
            dtv2=min(hpp2/(v2+epp),dtv2)
c  interpolation kernel and (1/r)(d/dr)
            rbin=rad2torbin*rad2
            ibin=int(rbin)
            fbin=rbin-ibin
            w(ispp)=wtab(ibin)+fbin*(wtab(ibin+1)-wtab(ibin))
            dw(ispp)=dwtab(ibin)+fbin*(dwtab(ibin+1)-dwtab(ibin))
c  density, velocity divergence, etc
            dbarpp=dbarpp+rmj*w(ispp)
#ifdef SPH2
            pbarpp=pbarpp+rmj*w(ispp)*e(j)
#endif
            df=rmj*dw(ispp)
#ifdef SPH2
            do k=1,3
               pgrad(k)=pgrad(k)+df*dr(k)*e(j)
            end do
#endif
            divppr=divppr-df*rdvr(ispp)
            divppc=divppc-df*rdvc(ispp)
#ifdef BALSARA
            dcvpp(1)=dcvpp(1)-df*rcv(1)
            dcvpp(2)=dcvpp(2)-df*rcv(2)
            dcvpp(3)=dcvpp(3)-df*rcv(3)
#endif
            xdwsum=xdwsum+rad2*df
         end do

c  normalise accumulated sph quantities     
         dbarpp=dbarpp*hpp3i
         dbarppi=1./dbarpp
#ifdef SPH2
c     NB the missing 2/3 in pbarpp is intentional 
c     --- it enables us to use epp rather than 2/3 epp in other expressions
         pbarpp = pbarpp*hpp3i
c  neither epp nor dpp should be zero --- if they are then a crash is best
         dpp = pbarpp/epp
         dppi = 1./dpp
#endif
         divppr=divppr*dbarppi*hpp5i
         divppc=divppc*dbarppi*hpp5i
#ifdef BALSARA
         dcvpp2=dcvpp(1)*dcvpp(1)+dcvpp(2)*dcvpp(2)+dcvpp(3)*dcvpp(3)
         dcvppm=sqrt(dcvpp2)*dni*hpp5i
         divppm=abs(divppr)
         balsara=divppm/(divppm+dcvppm+0.0001*cspp*hppi)
         if (balsara.lt.0.05) balsara=0.05
#endif
         fhvar=-3.*dbarpp/(xdwsum*hpp5i)
#ifndef NOHVARLOG
         fhvar=1.+log(fhvar)
#endif

c  second sph loop over neighbouring particles.
c  This does the accelerations and energy/entropy change.
c  See comment above first sph loop for difference between ispp & isppindex
         do ispp=1,nspp
#ifdef EXACTNSPH
            isppindex=index(ispp)
#else
            isppindex=ispp
#endif
            j=jlist(isppindex)
            rad2=radii2(isppindex)
            dr(1)=dx(1,isppindex)
            dr(2)=dx(2,isppindex)
            dr(3)=dx(3,isppindex)
            rdotv=rdvc(ispp)
            rmj=rm(j)
c
c  pressure force
#ifdef SPH1
            frcpress=twothirds*epp*dbarppi
#endif
#ifndef NOHVAR
            frcpress=frcpress*fhvar
#endif
#ifdef SPH2
#ifdef SPH2_NOCOMOVE
            frcpress=twothirds*e(j)*dppi
#else
            frcpress=twothirds*e(j)*dbarppi
#endif       
#endif
c
c  artificial viscosity
c     this is the old form based on the bulk div(v)
c     we have now decided that this is a bit too mushy
c            if (divppr.lt.0.) then
c               cspp=sqrt(tenninth*epp)
c               divpph=divppr*hpp
c               frc=frc+(bvisc*divpph-avisc*cspp)*divpph*dppi
c            endif
c     this is the pair-wise version
c     benefit: shorter-range
c     disadvantage: applies artificial viscosity even in diverging regions
            bviscuse=bvisc
c     we could use this condition to limit art visc in diverging regions
c     if (divppr*hpp .lt. 0.3*cspp) bviscuse=0.
            if (rdvr(ispp).lt.0.0) then
               csij = (cspp + sqrt(tenninth*e(j)))/2.
               rmupp=(hpp*rdvr(ispp))/((rad2+0.01*hpp2)*csij)
#ifdef BALSARA
               rmupp=rmupp*balsara
#endif
c     the artificial viscosity is symmetric in e(i) and e(j)
               frcart=onethird*(epp+e(j))*dbarppi
     &                 *(bviscuse*rmupp-avisc)*rmupp
            else
               frcart = 0.
            endif
c     
c     evaluate the force
            frc=frcpress+frcart
            frc2=frc*dw(ispp)*hpp5i
            frc3=frc2*rmj
c     on particle
            app(1)=app(1)+dr(1)*frc3
            app(2)=app(2)+dr(2)*frc3
            app(3)=app(3)+dr(3)*frc3
#ifdef TRACE
            if (i.eq.itest) write(*,'(i5,1p5e10.2)')
     &           j,e(j),(dr(k),k=1,3),frc3
#endif
c     on neighbours
            frc4=frc2*rmpp
            a(1,j)=a(1,j)-dr(1)*frc4  
            a(2,j)=a(2,j)-dr(2)*frc4  
            a(3,j)=a(3,j)-dr(3)*frc4  
c
c  change in energy
c  track separately that due to pressure and artificial viscosity 
            ffact = dw(ispp)*hpp5i*rmj
c  An object of constant physical size will have a net force in comoving 
c    co-ordinates; this leads to a heating that gets subtracted off
c    later in accel.  We do it this way so that particles in low-density
c    regions with few neighbours will still have the correct expansion cooling.
c  old way based on div(v) (check this before use)
c            depp=depp-frc*divppr*dpp
c  new way as an accumulation of pairwise terms
            frcpress=frcpress*ffact*rdotv
            depress=depress+frcpress
c  At first sight one might think that real co-ordinates should be used to
c  evaluate the change in energy from the artifical viscosity, ie
c            deart=deart+frcart*rdvr(ispp)
c  However, comoving co-ordinates seem to be better for two reasons:
c  (i) the Layzer-Irvine energy integral is conserved only if comoving
c      co-ordinates are used.
c  (ii) a hypothetical uniform artificial pressure (which could not exist
c       in practice except in universes that are contracting) will not
c       give rise to any net force and should not change the energy of
c       any particle (unlike a uniform real pressure which does do work
c       as the Universe expands and leads to a reduction in energy).
c       This is true only if we use comoving co-ordinates.
c  If anyone has any useful insights, then please let me know 
c  (p.a.thomas@sussex.ac.uk).
            frcart=frcart*ffact*rdotv
            deart=deart+frcart
#ifdef ENEIGH
c  try putting PdV work on neighbour
            de(j)=de(j)+frcpress*rmpp/rmj
#endif
         end do

c  new smoothing length
#ifdef EXACTNSPH
         h(i)=hpp*((1.+nspp)*sphni)**onethirdi
#else
#ifdef WEIGHTED_NEIGHBOUR
         h(i)=(2.*h(i)+hpp*((1.+wnspp)*sphni)**onethirdi)/3.
#else
         h(i)=(2.*h(i)+hpp*((1.+nspp)*sphni)**onethirdi)/3.
#endif
#endif
         h(i)=max(h(i),hmin)

c  new density (used in weighted-neighbour count and cooling)
#ifdef SPH1
         dn(i)=dbarpp
#endif
#ifdef SPH2
         pgradnorm=sqrt(pgrad(1)**2+pgrad(2)**2+pgrad(3)**2)*hpp5i
c  set density to unsmoothed value except in shocks
         if (pgradnorm*hpp.gt.qoverp*pbarpp) then
            dn(i)=dbarpp
         else
            dn(i)=dpp
         end if
#endif
#ifndef NOEXPAND
c  convert to overdensity
         dn(i)=dn(i)/rmbary
#endif

c  accumulate sph accelerations
         a(1,i)=a(1,i)+app(1)
         a(2,i)=a(2,i)+app(2)
         a(3,i)=a(3,i)+app(3)
#ifdef ENTROPY
         ds(i)=ds(i)+deart/dn(i)**twothirds
#endif
#ifdef ENEIGH
         de(i)=de(i)+deart
#else
         de(i)=de(i)+depress+deart
#endif

c  Debugging and diagnostic code
#ifdef DUMPHVAR
c  use h to store fhvar for diagnostic purposes
         h(i)=fhvar
#endif
#ifdef QOVERP
c  use h to store qoverp for diagnostic purposes
         h(i)=pgradnorm*hpp/pbarpp
#endif
#ifdef DEBUG
         if (i.eq.itest) then
c            print*,'i=',i
c            print*,'dn(i)=',dn(i)
c            print*,'dn0(i)=',dn0(i)
c            print*,'dnpp=',dnpp
c            print*,'dbarpp=',dbarpp
c            print*,'dpp=',dpp
c            print*,'e(i)=',e(i)
c            print*,'epp=',epp
c            print*,'pbarpp=',pbarpp
c            print*,'hdp/p=',pgradnorm*hpp/pbarpp
c            print*,'h(i)=',h(i)
c            print*,'hpp=',hpp
c            print*,'nspp=',nspp
c            print*,'wnspp=',wnspp
            open(20,name='fort.20',access='append')
            write(20,'(1p9e11.4)') dn(i),dbarpp,dpp,epp,de(i),pbarpp,
     &           pgradnorm*hpp/pbarpp,hpp,h(i)
            close(20)
         end if
#endif

c  diagnostic counters
         nstot=nstot+1
c  the nspppmax below is not a typo---nsppmax is the max allowed nspp
         nspppmax=max(nspppmax,nspp)
         nsppmin=min(nsppmin,nspp)
         nspptot=nspptot+nspp
#ifdef WEIGHTED_NEIGHBOUR
         wnsppmax=max(wnsppmax,wnspp)
         wnsppmin=min(wnsppmin,wnspp)
         wnspptot=wnspptot+wnspp
#endif
         fhvarmax=max(fhvarmax,fhvar)
         fhvarmin=min(fhvarmin,fhvar)
         fhvartot=fhvartot+fhvar

        end if
       end if
c     
       i=ll(i)
      end do
c  finished loop over particles in central box -------------------------
c     
c
            enddo
         enddo
      enddo
c  finished loop over central boxes^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
c
c  velocity timestep
      dtv=sqrt(dtv2)
c
c  write out disgnostics
#ifdef VERBOSE
      print*,'max nspp = ',nspppmax
      print*,'min nspp = ',nsppmin
      print*,'mean nspp = ',float(nspptot)/float(nstot)
#ifdef WEIGHTED_NEIGHBOUR
      print*,'max wnspp = ',wnsppmax
      print*,'min wnspp = ',wnsppmin
      print*,'mean wnspp = ',wnspptot/float(nstot)
#endif
#ifndef NOHVAR
      print*,'max fhvar = ',fhvarmax
      print*,'min fhvar = ',fhvarmin
      print*,'mean fhvar = ',fhvartot/float(nstot)
#endif
#endif
c
      RETURN
      end

c-----------------------------------------------------------------------
#ifdef EXACTNSPH
      FUNCTION select_index(k,n,arr,index)
c
c  On input, index should contain the values 1..n in order
c  Returns the k'th smallest element in the array arr(1:n) and does
c  a partial sort of the indexing array:  on return elements 1:k-1 are 
c  smaller than element k and elements k+1:n larger than element k.
c  The array arr is left intact (unsorted).
c
      integer k,n,index(n)
      real select_index,arr(n)
      integer i,ir,j,l,mid
      real a,temp
c
      l=1
      ir=n
1     if(ir-l.le.1)then
        if(ir-l.eq.1)then
          if(arr(index(ir)).lt.arr(index(l)))then
            itemp=index(l)
            index(l)=index(ir)
            index(ir)=itemp
          endif
        endif
        select_index=arr(index(k))
        RETURN
      else
        mid=(l+ir)/2
        itemp=index(mid)
        index(mid)=index(l+1)
        index(l+1)=itemp
        if(arr(index(l+1)).gt.arr(index(ir)))then
          itemp=index(l+1)
          index(l+1)=index(ir)
          index(ir)=itemp
        endif
        if(arr(index(l)).gt.arr(index(ir)))then
          itemp=index(l)
          index(l)=index(ir)
          index(ir)=itemp
        endif
        if(arr(index(l+1)).gt.arr(index(l)))then
          itemp=index(l+1)
          index(l+1)=index(l)
          index(l)=itemp
        endif
        i=l+1
        j=ir
        a=arr(index(l))
3       continue
          i=i+1
        if(arr(index(i)).lt.a)goto 3
4       continue
          j=j-1
        if(arr(index(j)).gt.a)goto 4
        if(j.lt.i)goto 5
        itemp=index(i)
        index(i)=index(j)
        index(j)=itemp
        goto 3
 5      itemp=index(l)
        index(l)=index(j)
        index(j)=itemp
        if(j.ge.k)ir=j-1
        if(j.le.k)l=i
      endif
      GOTO 1
      end
#endif
