      SUBROUTINE sph
c
      include 'psize.inc'
#include "rvarrays.inc"
      include 'pinfo.inc'
      include 'itype.inc'
c
c  rate at which search-cell changes in size)
      parameter (fracLg=0.5)
c
c  gridding arrays
      integer ll(Nmaxgas),ihc(Lgmax,Lgmax,Lgmax)
      real xmin(3)
c  control flags
      logical isolated

#ifdef EXACTNSPH
c  for exact number of sph neighbours must allow arbitrarily small h
      hmin=0.
#else
      hmin=0.5*soft
#endif
c
#ifdef WEIGHTED_NEIGHBOUR
c  store old densities in dn0 to make neighbour count scan-independent
      do i=1,Ngas
         dn0(i)=dn(i)
      end do
#endif
c
c  decide whether to wrap gas or not
      isolated=.false.
#ifdef ISOLATED
      isolated=.true.
#endif
c
c  Refinements: could put in refinements in a similar way to the gravity
c  in which case all that follows would be confined to a subroutine sph_grid
c     refinement=.false.
c     CALL rfinit_sph
c     CALL refine_sph
c     refinement=.true.
c     Loop_over_refinements
c        CALL sph_grid
c     end_loop
c
#ifdef VERBOSE
      print*,'Starting SPH calculation'
#endif
c
c  extent of gas particles
c     if refinement then
c        look up extent of refinement
c     else
      if (isolated) then
         CALL extent_gas(Ngas,itype,r,xmin,xsize)
      else
         do j=1,3
            xmin(j)=0.
         enddo
         xsize=1.
      end if
c     end if
c
c  Loop over range of search cell sizes.
c  Because we know the search length, h, it makes little difference whether
c    we search with large or small cells first.  The former fits more
c    naturally into the AP3M way of thinking.
c
c  define whether a padding cell is required
c      if (isolated .or. refinement)
      if (isolated) then
         Lgoff=1
      else
         Lgoff=0
      end if
c
c  find maximum search length required (=2hppmax) and count gas particles
      CALL searchmax(Ngas,itype,h,hppmax,icgas)
#ifdef VERBOSE
      print*,icgas,' unprocessed particles'
      print*,'hppmax =',hppmax
#endif
      sphtimetot=0.
c
c  Loop over search cell sizes
      Lgold=0
      do while (icgas.gt.0)
         sphtime0=cputime()
c
c  optimal cell-size so as to just find all neighbours
c  0.499 instead of 0.5 in case of rounding error
         Lg=Lgoff+int(0.499*xsize/hppmax)
c  need at least three and at most Lgmax cells in each direction
         Lg=min(Lgmax,max(3,Lg))
c  check that Lg != Lgold
         if (Lg.eq.Lgold) STOP 'sph: Lg=Lgold'
#ifdef VERBOSE
         print*,'Using Lg = ',Lg
#endif
         Lgold=Lg
c
c  decide upon minimum smoothing length that needs to be processed:
c  1) sorting into cells is O(N) so we can allow Lg to increase
c     slowly (this is different from gravity calculation)
c  2) knowing the distribution of particles in the current cells,
c     it must be possible to estimate the work for different Lg
c  3) with refinements, it would make sense to lower Lgmax and farm
c     the remaining particles out.
c  for now I will stick with a really simple estimate used by Frazer
c     which takes hppmin to be a constant fraction of the current cell-size
         hppmin=fracLg*0.5*xsize/float(Lg-Lgoff)
c  needs to be small enough to allow Lg to increase.
         hppmin=min(hppmin,0.498*xsize/float(Lg+1-Lgoff))
c  force evaluation of all remaining particles if Lg=Lgmax
         if (Lg.eq.Lgmax) hppmin=0.
#ifdef VERBOSE
         print*,'hppmin = ',hppmin
#endif
c
c  sort particles into search-cells
         xscale=float(Lg-Lgoff)/xsize
         CALL listg(Ngas,r,itype,ll,Lg,Lgoff,ihc,xmin,xscale)
c
c  calculate the sph forces
         CALL shsph(Ngas,Lg,ll,ihc,hmin,hppmin)
c
c  number and max smoothing length of remaining particles
         CALL searchmax(Ngas,itype,h,hppmax,icgas)
c     
         sphtime=cputime()-sphtime0
         sphtimetot=sphtimetot+sphtime
#ifdef VERBOSE
#ifdef CPUTIME
         print*,'loop took ',sphtime,' seconds'
#endif
         print*,icgas,' unprocessed particles'
         print*,'hppmax =',hppmax
#endif
c
c  end loop over cell-sizes
      end do
c
c  reset type of gas particles and check that all were processed
      do i=1,Ngas
         if (itype(i).eq.itgas) then
            print*,'sph: particle ',i,' not processed'
            STOP
         end if
         if (itype(i).eq.itdone) itype(i)=itgas
      enddo
#ifdef CPUTIME
      print*,'cputime sph     = ',sphtimetot
#endif
c
      RETURN
      end
c-----------------------------------------------------------------
c
      SUBROUTINE searchmax(N,itype,h,hppmax,icgas)
c
c  Counts the number of gas particles left and finds their maximum
c  smoothing length
c
      include 'itype.inc'
      integer itype(N)
      real*4 h(N)
c
      hppmax=0.
      icgas=0
      do i=1,N
         if (itype(i).eq.itgas) then
            icgas=icgas+1
            hppmax=max(hppmax,h(i))
         endif
      enddo
c
      RETURN
      end
