      SUBROUTINE startup
c  reads in control parameters and initial data
c
#include "check_options.inc"
c
      include 'psize.inc'
      include 'pinfo.inc'
#include "rvarrays.inc"
      include 'iodefs.inc'
      include 'itype.inc'
      include 'units.inc'
#ifdef SAVE_ACCEL
      logical save_accel
#endif
#ifdef MANUALREF
      include 'manualref.inc'
#endif
      real rmin(3)

#ifdef SAVE_ACCEL
      save_accel = .false.
      if(datafile(7:10).eq.'back'.or.datafile(7:10).eq.'data')
     &     save_accel=.true.
#endif

#ifdef MANUALREF
c  read in manual refinement list
      open(14,file='manualref.dat',status='old',form='formatted')
      read(14,*) nref
      do i=1,nref
      read(14,*) (mpsf(i,ii),ii=1,4)
      enddo
      close(14)
#endif

c  The order of the next three items is important________
c
c  read in name of datafile
      open(55,file='prun.dat',status='old')
      read(55,'(a10)') datafile
c
c  read in data
      CALL readdata
c
c  read in run control parameters and warn of changes
      CALL readparam
      close(55)

c  initialise output times
      icdump=0
      tout=-1.
      do while (tout.lt.time)
         icdump=icdump+1
  	 if (icdump.gt.icdumpmax) then
	    STOP 'startup: time exceeds output times requested'
         endif 
         tout=tout1(icdump)
      enddo

c  determine the co-ordinate range of the data
      CALL extent(Nobj,itype,r,rmin,rsize)
#ifdef ISOLATED
c  find centre of particle data
      do j=1,3
         rcen(j)=rmin(j)+0.5*rsize
      end do
c  if maximum radius is not already set, use the data extent
      if (rmax2.le.1e-30) rmax2=rsize**2 
#else
c  check that range is [0,1]
      do j=1,3
         if (rmin(j).lt.0. .or. rmin(j).gt.0.1)
     &        STOP 'data range must be [0,1] for periodic bc'
         rmin(j)=0.
      end do
      if (rsize.lt.0.9 .or. rsize.gt.1.0) 
     &     STOP 'data range must be [0,1] for periodic bc'
      rsize=1.
#endif

c check that all particles have sensible itypes
      do i=1,nobj
         if (itype(i).gt.ithigh) STOP 'STOP: itype too high'
         if (itype(i).lt.itlow) STOP 'STOP: itype too low'
      enddo

c  open log file
      write(lname,'(''pr'',i4.4,''.log'')')irun
      CALL openf_append(66,lname)

c  Do not move this next call too far down.  Many things need atime
c  evaluate expansion parameter
#ifndef NOEXPAND
      CALL ahtime(time,omega0,xlambda0,atime,htime)
#else
      atime=1.
      htime=0.	
#endif

c  set units
      print*
      CALL inunit

#ifdef NOSPH
      ngas=0
      ndark=nobj
#else
c  check that gas particles have their properties defined
c  (needs to be done after call to inunit)
      if (ngas.gt.0) then
      iwarnh=0
      iwarne=0
      iwarndn=0
c  if smoothing lengths not set, then reset to mean particle separation
      hmean=rsize/(float(ngas)**(1./3.))
c  if particle energies not set, then reset to 1K
      emin=1./Kunit/atime**2
      do i=1,ngas
         if (itype(i).eq.itgas) then
            if (h(i).lt.1.e-20) then
               iwarnh=1
               h(i)=hmean
            end if
            if (e(i).lt.emin) then
               iwarne=1
               e(i)=emin
            end if
            if (dn(i).lt.1e-20) then
               iwarndn=1
#ifdef NOEXPAND
               dn(i)=rm(i)/h(i)**3
#else
               dn(i)=(hmean/h(i))**3
#endif
            end if
         end if
      end do
      if (iwarnh.eq.1) 
     &     print*,'Warning: SPH particle smoothing lengths not set'
      if (iwarne.eq.1) 
     &     print*,'Warning: SPH particle temperatures not set'
      if (iwarndn.eq.1) 
     &     print*,'Warning: SPH particle densities not set'
      end if
#ifdef ENTROPY
c  convert the input temperatures into entropy
      do i=1,ngas
         s(i)=e(i)/dn(i)**(2./3.)
      end do
#endif
#endif

c  first evaluation of accel
      print*
      print*,'itime, time, atime, htime: ',itime,time,atime,htime
      cputo=cputime()
#ifdef SAVE_ACCEL
c  no need to evaluate accelerations if these have been read in
      if (.not.save_accel) CALL accel
#else
      CALL accel
#endif

#ifndef NOSPH
#ifdef ENTROPY
c  on first step reset entropies using new density estimate from accel
      if (itime.eq.0) then
         do i=1,ngas
#ifdef RELAX
c  make all the entropies the same to obtain a relaxed distribution
            s(i)=e(1)/dn(1)**(2./3.)
#else
            s(i)=e(i)/dn(i)**(2./3.)
#endif
         end do
      end if
#endif
#endif

c  initialise energy accounting
      CALL energyinit
      CALL infout
c
      RETURN
      end

c--------------------------------------------------------------
      SUBROUTINE energyinit
c
c  initialises energy accounting
c
      include 'psize.inc'
#include "rvarrays.inc"
      include 'pinfo.inc'
      include 'itype.inc'
      include 'units.inc'
c
      real*8 T8,Th8
c
      T8=0.
      Th8=0.
      do i=1,nobj
         if (itype(i).ne.itnone) then
            T8=T8+rm(i)*(v(1,i)**2+v(2,i)**2+v(3,i)**2)
#ifndef NOSPH
            if (itype(i).eq.itgas) Th8=Th8+rm(i)*e(i)
#endif
         endif
      end do
      T=0.5*T8*atime**2
      Th=Th8*atime**2
c
      RETURN
      end
