      SUBROUTINE updaterv
c
c  Calls accel and does PEC timestepping
c
c  alpha is the weighting given to the current a in the prediction step for r
      parameter(alpha=0.)
c
      include 'psize.inc'
#include "rvarrays.inc"
      include 'pinfo.inc'
      include 'itype.inc'
      include 'units.inc'
c
c  temporary accelerations
      real ao(3,Nmax),ad(3)
#ifndef NOSPH
      real deo(Nmaxgas)
#ifdef ENTROPY
      real dso(Nmaxgas)
#endif
#endif
c
      real*8 T8,Th8,Radiation8
#ifdef TEST
      parameter(deltaecut=0.5)
      real eoldtest(Nmaxgas)
      deltaemax=0.
      ideltaemax=0.
#endif
#ifdef TRACE
c  run1051
c      itest=1012
c  run1041
      itest=1502
#endif

c  remember current values for energy integrals
      atimeo=atime
      Uo=U
      To=T
      Tho=Th

c  rename saved accelerations
      do i=1,nobj
         ao(1,i)=a(1,i)
         ao(2,i)=a(2,i)
         ao(3,i)=a(3,i)
      end do
#ifndef NOSPH
      do i=1,ngas
         deo(i)=de(i)
#ifdef ENTROPY
         dso(i)=ds(i)
#endif
      end do
#endif

#ifdef TRACE
#ifdef ENTROPY
      print*,'1:',itest,dn(itest),e(itest),de(itest),s(itest),ds(itest)
#else
      print*,'1:',itest,dn(itest),e(itest),de(itest)
#endif
#endif

c  Predict..............................................................
      dt=dtime
      dto2=dt/2.
      dt2=dt**2
      dt2o2=dt2/2.
      dt2o6=dt2o2*alpha
#ifdef PHOTO_SS
      CALL photo(atime)
#else
      emin=1./Kunit/atime**2
#endif

      do i=1,nobj
       if (itype(i).ne.itnone) then

        do j=1,3
         r(j,i)=r(j,i)+v(j,i)*dt+ao(j,i)*dt2o2
#ifndef ISOLATED
c  periodic wrapping
         r(j,i)=mod(r(j,i)+10.,1.)
#endif
         v(j,i)=v(j,i)+ao(j,i)*dt
        end do

#ifndef NOSPH
        if (itype(i).eq.itgas) then
#ifdef TEST
         eoldtest(i)=e(i)
#endif
#ifdef ENTROPY
         s(i)=s(i)+dso(i)*dt
#endif
         e(i)=e(i)+deo(i)*dt
c  impose a minimum temperature on the gas
#ifdef PHOTO_SS
         dnpp=dn(i)*nunit/atime**3
         emin=max(1.,emin_ss(dnpp))/Kunit/atime**2
#endif
         e(i)=max(e(i),emin)
        end if
#endif

       end if
      end do

#ifdef TRACE
#ifdef ENTROPY
      print*,'2:',itest,dn(itest),e(itest),de(itest),s(itest),ds(itest)
#else
      print*,'2:',itest,dn(itest),e(itest),de(itest)
#endif
#endif

c  Evaluate...........................................................
#ifndef RELAX
      time=time+dt
#endif
#ifndef NOEXPAND
      CALL ahtime(time,omega0,xlambda0,atime,htime)
#else
      atime=1.
      htime=0.	
#endif
      itime=itime+1
      print*
      print*,'itime, time, atime, htime: ',itime,time,atime,htime
      CALL accel

#ifdef TRACE
#ifdef ENTROPY
      print*,'3:',itest,dn(itest),e(itest),de(itest),s(itest),ds(itest)
#else
      print*,'3:',itest,dn(itest),e(itest),de(itest)
#endif
#endif

c  Correct............................................................
      T8=0.
      Th8=0.
      Radiation8=0.
#ifdef PHOTO_SS
      CALL photo(atime)
#endif
#ifndef PHOTO_SS
      emin=1./Kunit/atime**2
#endif

      do i=1,nobj
       if (itype(i).ne.itnone) then

        do j=1,3
         ad(j)=a(j,i)-ao(j,i)
         v(j,i)=v(j,i)+ad(j)*dto2
         r(j,i)=r(j,i)+ad(j)*dt2o6
#ifndef ISOLATED
         r(j,i)=mod(r(j,i)+10.,1.)
#endif
        end do
        T8=T8+rm(i)*(v(1,i)**2+v(2,i)**2+v(3,i)**2)

#ifndef NOSPH
        if (itype(i).eq.itgas) then
#ifdef ENTROPY
         s(i)=s(i)+(ds(i)-dso(i))*dto2
c  convert entropy to energy
         dnfac=dn(i)**(2./3.)
c  on this corrector step we omit the predicted PdV work
         epp=s(i)*dnfac
#else
         epp=e(i)+(de(i)-deo(i))*dto2
#endif
         eold=epp
c  Cool the gas.
c  Done here so that the output routines have correct values in them.
c  Cooling should be done in tandem with heating but of course this is
c  all screwed up by shocks anyway.
         if(lcool.eq.1) then
#ifdef ZMETEVOL
          CALL cool(dn(i),epp,dt,zmet0*time)
#else
#ifdef ZMET
          CALL cool(dn(i),epp,dt,zmet(i))
#else
c  metallicity is irrelevant when ZMET is not defined (set in createcool)
          CALL cool(dn(i),epp,dt,zmet0)
#endif
#endif
          Radiation8=Radiation8+rm(i)*(eold-epp)
         end if
c  impose a minimum temperature
#ifdef PHOTO_SS
         dnpp=dn(i)*nunit/atime**3
         emin=max(1.,emin_ss(dnpp))/Kunit/atime**2
#endif
         e(i)=max(epp,emin)
         Th8=Th8+rm(i)*e(i)
#ifdef TEST
         deltae=abs(e(i)-eoldtest(i))/(e(i)+eoldtest(i))
         if (deltae.gt.deltaecut) 
     &        print*,'fractional energy change for ptcle ',
     &        i,' is ',deltae
         if (deltae.gt.deltaemax) then
            deltaemax=deltae
            ideltaemax=i
         end if
#endif
#ifdef ENTROPY
c  convert back to entropy
         s(i)=e(i)/dnfac
#endif
        end if
#endif

       end if
      end do
#ifdef TEST
      print*,'Maximum energy change is ',deltaemax,' for particle ',
     &     ideltaemax
#endif

#ifdef TRACE
#ifdef ENTROPY
      print*,'4:',itest,dn(itest),e(itest),de(itest),s(itest),ds(itest)
#else
      print*,'4:',itest,dn(itest),e(itest),de(itest)
#endif
#endif

c  update energy sums
      T=0.5*T8*atime**2
      Th=Th8*atime**2
      Radiation=Radiation8*atime**2
      dlnat=log(atime/atimeo)
      Esum=Esum+0.5*(2.*(T+Th)-U + 2.*(To+Tho)-Uo)*dlnat
      Rsum=Rsum+radiation

c  accounting
      CALL infout

#ifdef ISOLATED
c  Check for particles leaving the (spherical) box and sum lost energies
c  Done here so as not to invalidate energy sums
      do i=1,nobj
       if (itype(i).ne.itnone) then
        if ((r(1,i)-rcen(1))**2+(r(2,i)-rcen(2))**2
     &   +(r(3,i)-rcen(3))**2 .gt. rmax2) then
         if (itype(i).eq.itgas) then
            epp=e(i)
         else
            epp=0.
         endif
         CALL lostp(nobj,i,rm,r,v(1,i),epp,itype,soft,rmnorm0,atime,
     &        Tlost,Qlost,dUlost)
         Ulost=Ulost+dUlost
        end if
       end if
      end do
#endif

      RETURN
      end
