      program convert

c ---------------------------------------------------------------------
c     name: convert.F
c     begun: 24/11/99
c     last change: 19/1/2000
c     last change by: Ben
c     description: converts between hydra3 and 4 format (or vice versa)
c     uses: readdata.F dumpdata.F system.f (hydra4.0 versions)
c     known bugs: none (bug reports to benr@astr.cpes.susx.ac.uk)
c ---------------------------------------------------------------------
c     notes: I've decided not to try and autodetect file formats, 
c            although I guess it should be possible. As things stand 
c            the default is to read H3 and write H4, with the -h3 flag
c            reversing the process. If you try and read an H4 file
c            without the -h3 flag it will probably just fall over.
c            
c            This hasn't been tested with the -DZMET extended data files
c            (if someone does try it, let me know what happens!). 
c     
c            The osarg parsing isn't exactly robust, in case you hadn't
c            noticed.
c ---------------------------------------------------------------------


      include 'psize.inc'
      include 'pinfo.inc'
      include 'iodefs.inc'
#include "rvarrays.inc"
      include 'itype.inc'

      logical verbose,ofname,new2old
      character*80 osarg,infile,outfile

      verbose = .false.
      ofname = .false.
      new2old = .false.

c     parse the iargs(). 

c     if there are no args (or more than 4) then just display usage and quit
      if ((iargc().eq.0).or.(iargc().gt.4)) then
         call hdisp()
c     one arg could be either a filename or a flag
c     (but just a flag is meaningless at the moment)
      else if (iargc().eq.1) then
         call getarg(1,osarg)
c     if osarg(1:1) is - then the arg must be a flag, else it is a filename
c     if it is a flag then we display usage and quit
         if (osarg(1:1).eq.'-') then
            call hdisp()
         else
            infile = osarg
            outfile = infile(1:5)//'.new'
            ofname = .true.
         endif
c     now the permutations start to build up. Two args
      else if (iargc().eq.2) then
         call getarg(1,osarg)
         if (osarg(1:1).eq.'-') then
            call parsearg(osarg,verbose,new2old)
            call getarg(2,infile)
            outfile = infile(1:5)//'.new'
            ofname = .true.            
         else 
            infile = osarg
            call getarg(2,outfile)
         endif
c     Three args
      else if (iargc().eq.3) then
         call getarg(1,osarg)
         if (osarg(1:1).ne.'-') then
            call hdisp()
         else
            call parsearg(osarg,verbose,new2old)
         endif
         call getarg(2,osarg)
         if (osarg(1:1).eq.'-') then
            call parsearg(osarg,verbose,new2old)
            call getarg(3,infile)
            outfile = infile(1:5)//'.new'
            ofname = .true.            
         else 
            infile = osarg
            call getarg(3,outfile)
         endif
c     Four args. Rather than parse them, we'll just assume they are correct
c     (i.e. -v -h3 infile outfile).
      else
         verbose = .true.
         new2old = .true.
         call getarg(3,infile)
         call getarg(4,outfile)
      endif

c     dont allow the input data to be overwritten
      if (infile.eq.outfile) STOP 'infile = outfile somehow'

c ---- read in data
      
      if (new2old) then
         datafile = infile
         call readdata
      else
         ier1 = 99
         Ndum = 1
         lun = 11
         call h3readdata(infile,lun,ier1,verbose,Nmax,Ndum,rm,
     &        r,v,h,dn,e,itype,zmet)
         call h3readdata(infile,lun,ier1,verbose,Nmax,nobj,rm,
     &        r,v,h,dn,e,itype,zmet)
         if (ier1.ne.0) then
 1999       format(' ERROR reading from: ',a20)
            write(*,1999) infile
            STOP
         endif
      endif

c     print out diagnostics if required. 

      if (verbose) then 
         if (new2old) then
            write(*,*) 'Converting from Hydra4 format to Hydra3 format'
         else
            write(*,*) 'Converting from Hydra3 format to Hydra4 format'
         endif
 997     format(' Read',i6,' particles from ',a20)
 996     format(' (ngas: ',i6,' ndark: ',i6,' time: ',f5.3, 
     &        ', timestep: ',i5,')')
         write(*,997) nobj,infile
         write(*,996) ngas, ndark, time,itime
      endif
      if (verbose.and.ofname) then
 998     format(' output file: ', a20)
         write(*,998) outfile
      endif

      if (ofname.eqv..false.) then
         read(outfile(2:5),*) irun
 995     format(' setting irun = ', i5)
         if (verbose) write(*,995) irun
      endif
         
c ---- write out data

      datafile = outfile(1:10)

      if (new2old) then
         ier = 99
         lun = 12
         CALL h3dumpdata(datafile,lun,ier,nobj,rm,r,v,h,dn,e,
     &        itype,zmet)
         if (ier.ne.0) STOP 'error writing out file'
      else
         CALL dumpdata
      endif
      end
      
      subroutine hdisp()
      write(*,*) 'usage convert [-v] input_filename [output_filename]'
      write(*,*)
      write(*,*) '      v - verbose'
      write(*,*)
      stop
      end

      subroutine parsearg(osarg,verbose,new2old)
      character*80 osarg
      logical verbose,new2old
      
      if (osarg(2:2).eq.'v') then
         verbose = .true.
      else if (osarg(2:2).eq.'h') then
         new2old = .true.
      else
         write(*,*) 'Unknown option'
         call hdisp()
      endif
      return
      end

c ============================================================
c I'm not sure if there is a better way to do this - we need to read
c the old H3 format common blocks in one read statement
c read(lun) ibuf_,ibuf1_,ibuf2_
c It's rather ugly.

      SUBROUTINE h3readdata(inputfile,lun,ier,verbose,Nmax,N,rm,r,
     &     v,h,dn,e,itype,zmet)

c H3 pinfo.inc common block -----------------------

      common/param_/itime_,itstop_,itdump_,itout_,  
     &     time_,atime_,htime_,dtime_,
     &     Est_,T_,Th_,U_, 
     &     Radiation_,Esum_,Rsum_,cpu_,
     &     tstop_,tout_,icdump_,padding_,  
     &     Tlost_,Qlost_,Ulost_
      common/start_/irun_,nobj_,ngas_,ndark_,
     &     L_,intl_,nlmx_,perr_,
     &     dtnorm_,sft0_,sftmin_,sftmax_,  
     &     h100_,box100_,zmet0_,spc0_,
     &     rmbary_,rmgas_,rmdark_,rmnorm_, 
     &     tstart_,omega0_,xlambda0_,h0t0_,lcool_
      common/outputtime_/tout1_
      real tout1_(25)
      dimension ibuf1_(100),ibuf2_(100),ibuf_(200)
      equivalence (ibuf1_,itime_),(ibuf2_,irun_),(ibuf_,tout1_)

c -------------------------------------------------
c include the H4 common block

      include 'pinfo.inc'
      include 'itype.inc'

c -------------------------------------------------
c define arrays

      character*60 inputfile,longfile
      logical verbose,gasfirst
c
      real rm(N),r(3,N),v(3,N),dn(N),e(N),h(N)
#ifdef ZMET
      real zmet(N)
#endif
      integer itype(N)

c
c ------------------------------------------------
c now read in the H3 header

      iflag=ier
      write(longfile,'(''data/'',a10)')inputfile 
      open(lun,file=longfile,status='old',form='unformatted',iostat=ier)
      if (ier.ne.0) RETURN

      read(lun) ibuf_,ibuf1_,ibuf2_

c ------------------------------------------------------------
c now for the ugly bit
c N.B L_, spc0_, rmbary_, rmgas_, rmdark_ and rmnorm_ have no
c equivalent in the H4 common blocks, so they get discarded

      itime = itime_
      itstop = itstop_
      itdump = itdump_
      itout = itout_
      time = time_
      atime = atime_
      htime = htime_
      dtime = dtime_
      Est = Est_
      T = T_
      Th = Th_
      U = U_
      Radiation = Radiation_
      Esum = Esum_
      Rsum = Rsum_
      cputo = cpu_
      tstop = tstop_
      tout = tout_
      icdump = icdump_
      Tlost = Tlost_
      Qlost = Qlost_
      Ulost = Ulost_

      irun = irun_
      nobj = nobj_
      ngas = ngas_
      ndark = ndark_
      intl = intl_
      nlmx = nlmx_
      perr = perr_
      dtnorm = dtnorm_
      sft0 = sft0_
      sftmin = sftmin_
      sftmax = sftmax_
      h100 = h100_
      box100 = box100_
      zmet0 = zmet0_
      lcool = lcool_
      tstart = tstart_
      omega0 = omega0_
      xlambda0 = xlambda0_
      h0t0 = h0t0_

c     change output times (tout1 has 50 elements in H4, 25 in H3)
      do i=1,25
         tout1(i)=tout1_(i)
      end do
      do i=26,50
         tout1(i) = 0.0
      end do

c ------------------------------------------------------------
c extras used by h4 - variables are set to zero,
c rather than being left undefined
c
      pad1 = 0.
      pad2 = 0.
      pad3 = 0.
      pad4 = 0.
      pad5 = 0.

c evaluated in accel.F and shsph.F 
      soft2 = 0.
      hmin = 0. 
      rmnorm = 0.
      dtv = 0.
      dta = 0.
      dtcs = 0.

c evaluated in inunit.F
      rmnorm0 = 0.
      rmbary = 0.    

c evaluated in startup.F

      rcen(1) = 0.5
      rcen(2) = 0.5
      rcen(3) = 0.5
      rmax2 = 0.

c ------------------------------------------------------------
c first sanity checks

      if(nobj.gt.Nmax) then
         write(*,*) nobj,Nmax
         STOP 'readdata: N>Nmax'
      endif
      if (iflag.eq.99)  then
         close(lun)
      return
      endif
      if(nobj.gt.Nmax) STOP 'readdata: N>Nmax'
c
c ------------------------------------------------------------
c now read in the assorted arrays

      read(lun) rm
      read(lun) r
      read(lun) v
      read(lun) h
      read(lun) e
      read(lun) itype
      read(lun) dn
#ifdef ZMET
      read(lun,end=1) zmet
      goto 2
 1    continue
      do i=1,nobj
         zmet(i)=zmet0
      enddo
 2    continue
#endif

c ------------------------------------------------------------
c second sanity checks

      do i=1,nobj
         if (itype(i).gt.ithigh) STOP 'STOP: itype too high'
         if (itype(i).lt.itlow) STOP 'STOP: itype too low'
      enddo

c ------------------------------------------------------------
c we need gas particles to be first - arrays like h() have size
c ngas in H4, not nobj as in H3. First check to see if this is
c the case

      if (ndark.gt.0) then
         gasfirst = .true.
         do i=1,nobj
            if ((itype(i).lt.itgas).and.(i.lt.ngas)) 
     &           gasfirst=.false.
         end do

         if (gasfirst) then
            if (verbose) write(*,*) 'Quicksorting Gas/DM'
         else
            if (verbose) write(*,*) 'Quicksorting gas....'
            call sort(nobj,rm,r,v,h,dn,e,itype,zmet)
         endif
      endif
c done
      
      RETURN
      end


c ============================================================

      SUBROUTINE h3dumpdata(outfile,lun,ier,N,rm,r,v,h,dn,e,
     &     itype,zmet)
c
c  Again, a modified version of the Hydra3 file.
c
c H3 pinfo.inc common block -----------------------

      common/param_/itime_,itstop_,itdump_,itout_,  
     &     time_,atime_,htime_,dtime_,
     &     Est_,T_,Th_,U_, 
     &     Radiation_,Esum_,Rsum_,cpu_,
     &     tstop_,tout_,icdump_,padding_,  
     &     Tlost_,Qlost_,Ulost_
      common/start_/irun_,nobj_,ngas_,ndark_,
     &     L_,intl_,nlmx_,perr_,
     &     dtnorm_,sft0_,sftmin_,sftmax_,  
     &     h100_,box100_,zmet0_,spc0_,
     &     rmbary_,rmgas_,rmdark_,rmnorm_, 
     &     tstart_,omega0_,xlambda0_,h0t0_,lcool_
      common/outputtime_/tout1_
      real tout1_(25)
      dimension ibuf1_(100),ibuf2_(100),ibuf_(200)
      equivalence (ibuf1_,itime_),(ibuf2_,irun_),(ibuf_,tout1_)

c -------------------------------------------------
c include the H4 common block

      include 'pinfo.inc'
      include 'itype.inc'

c -------------------------------------------------
c define arrays

      character*60 outfile,longfile
c
      real rm(N),r(3,N),v(3,N),dn(N),e(N),h(N)
#ifdef ZMET
      real zmet(N)
#endif
      integer itype(N)
c -------------------------------------------------
c now for another ugly bit

      itime_ = itime
      itstop_ = itstop
      itdump_ = itdump
      itout_ = itout
      time_ = time
      atime_ = atime
      htime_ = htime
      dtime_ = dtime
      Est_ = Est
      T_ = T
      Th_ = Th
      U_ = U
      Radiation_ = Radiation
      Esum_ = Esum
      Rsum_ = Rsum
      cputo_ = cpu
      tstop_ = tstop
      tout_ = tout
      icdump_ = icdump
      Tlost_ = Tlost
      Qlost_ = Qlost
      Ulost_ = Ulost

      irun_ = irun
      nobj_ = nobj
      ngas_ = ngas
      ndark_ = ndark
      intl_ = intl
      nlmx_ = nlmx
      perr_ = perr
      dtnorm_ = dtnorm
      sft0_ = sft0
      sftmin_ = sftmin
      sftmax_ = sftmax
      h100_ = h100
      box100_ = box100
      zmet0_ = zmet0
      lcool_ = lcool
      tstart_ = tstart
      omega0_ = omega0
      xlambda0_ = xlambda0
      h0t0_ = h0t0

c     change output times (tout1 has 50 elements in H4, 25 in H3)
c     if tout(26) is zero then copy directly, else average them
c     (because I can't think of a better thing to do with them)

      if (tout1(26).eq.0.) then 
         do i=1,25
            tout1_(i)=tout1(i)
         end do
      else
         icount = 1
         do i=1,49,2
            toutx = tout1(i)
            touty = tout1(i+1)
            tout1_(icount)=(toutx+touty)/2.
            icount = icount + 1
         end do
      endif
c     sanity check, to stop tout decreasing due to being averaged
c     with zero
      do i=1,24
         if  ((tout1_(i+1).lt.tout1_(i)).
     &        and.(tout1_(i+1).ne.0.)) then
            tout1_(i+1)=tout1_(i)+0.01
         endif
      end do

c ------------------------------------------------------------
c extras used by h3 - variables are set to zero,
c rather than being left undefined
c
      L = 0.
      spc0 = 0.
      rmbary = 0.
      rmgas = 0.
      rmdark = 0.
      rmnorm = 0.
      
c ------------------------------------------------------------
c now have h,e and dn for dark matter, too. Fill with zeros
c (we dont need to do any searching because we know that the
c particles from ngas+1 -> nobj are dark matter

      if (nobj.gt.ngas) then
         do i=ngas+1,nobj
            h(i) = 0.
            e(i) = 0.
            dn(i) = 0.
c presumably we also need to do this?
#ifdef ZMET
            zmet(i) = 0.
#endif
         end do
      end if

c ------------------------------------------------------------
c all done. Now write
     

c
      write(longfile,'(''data/'',a10)')outfile
      open(lun,file=longfile,status='unknown',form='unformatted'
     &     ,iostat=ier)
      write(lun) ibuf_,ibuf1_,ibuf2_
      write(lun) rm
      write(lun) r
      write(lun) v
      write(lun) h
      write(lun) e
      write(lun) itype
      write(lun) dn
#ifdef ZMET
      write(lun) zmet
#endif
      close(lun)
c
      RETURN
      end


c ============================================================

      SUBROUTINE sort(n,rm,r,v,h,dn,e,itype,zmet)
    
      include 'psize.inc'
      include 'itype.inc'

      integer n, iwksp(nmax),itype(n),itwksp(nmax),irz(nmax)
      real rm(n),r(3,n),v(3,n),h(n),dn(n),e(n),wksp(nmax)
#ifdef ZMET
      real zmet(n)
#endif
c     based on Numerical Recepies Ch.8 P332

c     require sorting by itype
c     it's a bit of a hack, but saves rewriting the quicksort
      do i=1,n
         if (itype(i).eq.itgas) then 
            irz(i) = -999
         else
            irz(i) = itype(i)
         endif
      end do

      CALL indexx(n,irz,iwksp)

c now copy

      do i=1,n
         itwksp(i)=itype(i)
      end do
      do i=1,n
         itype(i)=itwksp(iwksp(i))
         if (itype(i).eq.-999) itype(i)=itgas
      end do

c now do the rest

      do i=1,n
         wksp(i)=rm(i)
      end do
      do i=1,n
         rm(i)=wksp(iwksp(i))
      end do

      do j=1,3
         do i=1,n
            wksp(i)=r(j,i)
         end do
         do i=1,n
            r(j,i)=wksp(iwksp(i))
         end do
      end do

      do j=1,3
         do i=1,n
            wksp(i)=v(j,i)
         end do
         do i=1,n
            v(j,i)=wksp(iwksp(i))
         end do
      end do
      
      do i=1,n
         wksp(i)=h(i)
      end do
      do i=1,n
         h(i)=wksp(iwksp(i))
      end do

      do i=1,n
         wksp(i)=dn(i)
      end do
      do i=1,n
         dn(i)=wksp(iwksp(i))
      end do

      do i=1,n
         wksp(i)=e(i)
      end do
      do i=1,n
         e(i)=wksp(iwksp(i))
      end do

#ifdef ZMET
      do i=1,n
         wksp(i)=zmet(i)
      end do
      do i=1,n
         zmet(i)=wksp(iwksp(i))
      end do
#endif
      
c done 

      return
      END

c ============================================================
c     From Numerical Recepies

      SUBROUTINE indexx(n,arr,indx)

      include 'psize.inc'
      
      integer n,indx(Nmax),M,NSTACK
      integer arr(n)
      parameter (m=7,nstack=50)
      integer i,indxt,ir,itemp,j,jstack,k,l,istack(NSTACK)
      real a
      do j=1,n
         indx(j)=j
      end do
      jstack =  0
      l = 1
      ir = n
 1    if (ir-l.lt.M) then
         do j=l+1,ir
            indxt=indx(j)
            a=arr(indxt)
            do i=j-1,1,-1
               if(arr(indx(i)).le.a) goto 2
               indx(i+1)=indx(i)
            end do
            i=0
 2          indx(i+1)=indxt
         end do
         if (jstack.eq.0) return
         ir = istack(jstack)
         l = istack(jstack-1)
         jstack=jstack-2
      else
         k=(l+ir)/2
         itemp=indx(k)
         indx(k)=indx(l+1)
         indx(l+1)=itemp
         if (arr(indx(l+1)).gt.arr(indx(ir))) then
            itemp=indx(l+1)
            indx(l+1)=indx(ir)
            indx(ir)=itemp
         endif
         if (arr(indx(l)).gt.arr(indx(ir))) then
            itemp=indx(l)
            indx(l)=indx(ir)
            indx(ir)=itemp
         endif
         if (arr(indx(l+1)).gt.arr(indx(l))) then
            itemp = indx(l+1)
            indx(l+1)=indx(l)
            indx(l)=itemp
         endif
         i=l+1
         j=ir
         indxt=indx(l)
         a=arr(indxt)
 3       continue
         i=i+1
         if (arr(indx(i)).lt.a) goto 3
 4       continue
         j=j-1
         if(arr(indx(j)).gt.a) goto 4
         if (j.lt.i) goto 5
         itemp=indx(i)
         indx(i)=indx(j)
         indx(j)=itemp
         goto 3
 5       indx(l)=indx(j)
         indx(j)=indxt
         jstack=jstack+2
         if (jstack.gt.NSTACK) pause 'NSTACK too small in indexx'
         if (ir-i+1.ge.j-l)then
            istack(jstack)=ir
            istack(jstack-1)=i
            ir=j-1
         else
            istack(jstack)=j-1
            istack(jstack-1)=l
            l=i
         endif
      endif
      goto 1
      end


