      program hydra2tipsy
c
c     converts Hydra binary files to tipsy binary format
c     no longer works with g77 but I can't work out why
c
      include 'psize.inc'
      include 'pinfo.inc'
#include "rvarrays.inc"
      include 'iodefs.inc'
      include 'units.inc'
      include 'itype.inc'
c
c      parameter(nbuf=max(12,Nmax/8))
      parameter(nbuf=Nmax/8)
      real realbuf(nbuf)
      integer intbuf(nbuf)
      double precision dbtime
      equivalence (realbuf,intbuf),(realbuf,dbtime)
      integer index(Nmax)
c
 1    write(6,'(''Input Hydra file: '',$)')
      read(5,'(a)') datafile
c
      CALL readdata
      print*,'h2t: r(1:3,1)=',(r(j,1),j=1,3)
c
      call inunit
c
#ifdef NOEXPAND
c Mass in Msun
c distance in Mpc
      rscale=lunit/Mpc
      print*,'rscale=',rscale
c velocity in km/s
      vscale=vunit/1000.
      print*,'vscale=',vscale
c Temperature in Kelvin
      tscale=Kunit
      print*,'tscale=',tscale
c Number density per cm^3
      dscale=nunit
      print*,'dscale=',dscale
#else
      CALL ahtime(time,omega0,xlambda0,atime,htime)
c Mass in Msun
c distance in h-1Mpc
      rscale=lunit*atime/Mpc*h100
c velocity in km/s
      vscale=vunit*atime/1000.
c Temperature in Kelvin
      tscale=Kunit*atime**2
c Number density relative to mean density in box
      dscale=1.
#endif
      call bin_open(irun,itime)
c
      print*,'read  ',datafile
c
c     number of stars
      ngas=0
      ndark=0
      nstar=0
      iobj=0
#ifdef NSKIP
      nloop=0
      ilp=nint(nobj**(1./3.))
      if (ilp**3.eq.nobj) then
         nloop=1
      else if (ilp**3.eq.nobj/2) then
         nloop=2
         ilp=nint((nobj/2)**(1./3.))
      end if
      do iloop=1,nloop
         ilp2=ilp**2
         do iz=0,ilp-1,NSKIP
            izi=iz*ilp2
            do iy=0,ilp-1,NSKIP
               iyi=izi+iy*ilp
               do ix=1,ilp,NSKIP
                  i=iyi+ix
                  iobj=iobj+1
                  index(iobj)=i
                  if (itype(i).eq.ITGAS) then
                     ngas=ngas+1
                  else if(itype(i).eq.ITDARK) then
                     ndark=ndark+1
                  else if (itype(i).eq.ITSTAR) then
                     nstar=nstar+1
                  end if
               enddo
            enddo
         enddo
      enddo
      if (iobj.eq.0) then
         iseed=-12321
         fraction=1./NSKIP**3
         do i=1,nobj
            if (RAN3(iseed).lt.fraction) then
               iobj=iobj+1
               index(iobj)=i
               if (itype(i).eq.ITGAS) then
                  ngas=ngas+1
               else if(itype(i).eq.ITDARK) then
                  ndark=ndark+1
               else if (itype(i).eq.ITSTAR) then
                  nstar=nstar+1
               end if
            end if
         enddo
      end if
#else
      do i=1,nobj
         index(i)=i
         if (itype(i).eq.ITGAS) then
            ngas=ngas+1
         else if(itype(i).eq.ITDARK) then
            ndark=ndark+1
         else if (itype(i).eq.ITSTAR) then
            nstar=nstar+1
         end if
      enddo
      iobj=nobj
#endif
c
c --------- tipsy binary format --------
c Header: 
c  time(double), ntot, ndim, nsph(ngas), ndark, nstar, 0
c                       (last 0 presumably an alignment becasue of double time)
c for each ngas:
c  mass,x,y,z,v_x,v_y,v_z,rho,T,h,Z,phi (12 words) 
c for each ndark:
c  mass,x,y,z,v_x,v_y,v_z,soft,phi (9 words)
c for each nstar
c  mass, x,y,z,v_x,v_y,v_z,Z,t_form,soft,phi (11 words)
c
c
      print*,'Writing data (nbuf = ',nbuf,')'
c Header
      dbtime=0.d0
      dbtime=dbtime+time
      intbuf(3)=ngas+ndark+nstar
      intbuf(4)=3
      intbuf(5)=ngas
      intbuf(6)=ndark
      intbuf(7)=nstar
#ifndef linux
      intbuf(8)=0
c
      len=8
#else
      len=7
#endif
      call bin_write(intbuf,len)
c
      ioff=0
      ic=0
      do j=1,iobj
       i=index(j)
       if (itype(i).eq.ITGAS) then
        realbuf(ioff+1)=rm(i)*munit
        realbuf(ioff+2)=r(1,i)*rscale
        realbuf(ioff+3)=r(2,i)*rscale
        realbuf(ioff+4)=r(3,i)*rscale
        realbuf(ioff+5)=v(1,i)*vscale
        realbuf(ioff+6)=v(2,i)*vscale
        realbuf(ioff+7)=v(3,i)*vscale
        realbuf(ioff+8)=dn(i)*dscale
        realbuf(ioff+9)=e(i)*tscale
        realbuf(ioff+10)=h(i)*rscale
        realbuf(ioff+11)=0.
        realbuf(ioff+12)=0.
        ioff=ioff+12
        ic=ic+1
c
        if(ioff+12.gt.nbuf .or. ic.eq.ngas)then
           len=ioff
           call bin_write(intbuf,len)
           ioff=0
        end if
c
       end if
      enddo
      print*,'Done gas: ngas=',ic
c
      ioff=0
      ic=0
      do j=1,iobj
       i=index(j)
       if (itype(i).eq.ITDARK) then
        realbuf(ioff+1)=rm(i)*munit
        realbuf(ioff+2)=r(1,i)*rscale
        realbuf(ioff+3)=r(2,i)*rscale
        realbuf(ioff+4)=r(3,i)*rscale
        realbuf(ioff+5)=v(1,i)*vscale
        realbuf(ioff+6)=v(2,i)*vscale
        realbuf(ioff+7)=v(3,i)*vscale
        realbuf(ioff+8)=sft0*rscale
        realbuf(ioff+9)=0.
        ioff=ioff+9
        ic=ic+1
c
        if(ioff+9.gt.nbuf .or. ic.eq.ndark)then
           len=ioff
           call bin_write(intbuf,len)
           ioff=0
        end if
       end if
      enddo
      print*,'Done dark: ndark=',ic
c
      ioff=0
      ic=0
      do j=1,iobj
       i=index(j)
       if (itype(i).eq.ITSTAR) then
        realbuf(ioff+1)=rm(i)*munit
        realbuf(ioff+2)=r(1,i)*rscale
        realbuf(ioff+3)=r(2,i)*rscale
        realbuf(ioff+4)=r(3,i)*rscale
        realbuf(ioff+5)=v(1,i)*vscale
        realbuf(ioff+6)=v(2,i)*vscale
        realbuf(ioff+7)=v(3,i)*vscale
        realbuf(ioff+8)=0.
        realbuf(ioff+9)=0.
        realbuf(ioff+10)=0.
        realbuf(ioff+11)=0.
        ioff=ioff+11
        ic=ic+1
c
        if(ioff+11.gt.nbuf .or. ic.eq.nstar)then
           len=ioff
           call bin_write(intbuf,len)
           ioff=0
        end if
       end if
      enddo
      print*,'Done stars: nstar=',ic
c
      call bin_close()
c
      stop
      end
c-----------------------------------------------------------------
      real FUNCTION ran3(idum)
c  portable random number generator
c         implicit real*4(m)
c         parameter (mbig=4000000.,mseed=1618033.,mz=0.,fac=2.5e-7)
      parameter (mbig=1000000000,mseed=161803398,mz=0,fac=1.e-9)
c
c     integer idum,mbig,mseed,mz,ma(55),iff,mj,mk,inext,inextp
      integer ma(55)
c     real fac
      save ma,inext,inextp,iff
      data iff /0/
c
      if(idum.lt.0.or.iff.eq.0)then
        iff=1
        mj=mseed-iabs(idum)
        mj=mod(mj,mbig)
        ma(55)=mj
        mk=1
        do 11 i=1,54
          ii=mod(21*i,55)
          ma(ii)=mk
          mk=mj-mk
          if(mk.lt.mz)mk=mk+mbig
          mj=ma(ii)
11      continue
        do 13 k=1,4
          do 12 i=1,55
            ma(i)=ma(i)-ma(1+mod(i+30,55))
            if(ma(i).lt.mz)ma(i)=ma(i)+mbig
12        continue
13      continue
        inext=0
        inextp=31
        idum=1
      endif
      inext=inext+1
      if(inext.eq.56)inext=1
      inextp=inextp+1
      if(inextp.eq.56)inextp=1
      mj=ma(inext)-ma(inextp)
      if(mj.lt.mz)mj=mj+mbig
      ma(inext)=mj
      ran3=mj*fac
      RETURN
      end
